# -*- encoding: utf-8 -*-
from abjad import *


def test_lilypondfiletools_LilyPondFile___format___01():

    staff = Staff("c'8 d'8 e'8 f'8")
    lilypond_file = lilypondfiletools.make_basic_lilypond_file(staff)
    lilypond_file.file_initial_system_comments = []
    lilypond_file.file_initial_system_includes = []
    lilypond_file.file_initial_user_comments.append('User comments 1.')
    lilypond_file.file_initial_user_comments.append('User comments 2.')
    lilypond_file.file_initial_user_includes.append('external-settings-file-1.ly')
    lilypond_file.file_initial_user_includes.append('external-settings-file-2.ly')
    lilypond_file.default_paper_size = 'letter', 'portrait'
    lilypond_file.global_staff_size = 16
    lilypond_file.header_block.composer = markuptools.Markup('Josquin')
    lilypond_file.header_block.title = markuptools.Markup('Missa sexti tonus')
    lilypond_file.layout_block.indent = 0
    lilypond_file.layout_block.left_margin = 15
    lilypond_file.paper_block.oddFooterMarkup = markuptools.Markup('The odd-page footer')
    lilypond_file.paper_block.evenFooterMarkup = markuptools.Markup('The even-page footer')

    r'''
    % User comments 1.
    % User comments 2.

    \include "external-settings-file-1.ly"
    \include "external-settings-file-2.ly"

    #(set-default-paper-size "letter" 'portrait)
    #(set-global-staff-size 16)

    \header {
        composer = \markup { Josquin }
        title = \markup { Missa sexti tonus }
    }

    \layout {
        indent = #0
        left-margin = #15
    }

    \paper {
        evenFooterMarkup = \markup { The even-page footer }
        oddFooterMarkup = \markup { The odd-page footer }
    }

    \new Staff {
        c'8
        d'8
        e'8
        f'8
    }
    '''

    assert systemtools.TestManager.compare(
        lilypond_file,
        r'''
        % User comments 1.
        % User comments 2.

        \include "external-settings-file-1.ly"
        \include "external-settings-file-2.ly"

        #(set-default-paper-size "letter" 'portrait)
        #(set-global-staff-size 16)

        \header {
            composer = \markup { Josquin }
            title = \markup { Missa sexti tonus }
        }

        \layout {
            indent = #0
            left-margin = #15
        }

        \paper {
            evenFooterMarkup = \markup { The even-page footer }
            oddFooterMarkup = \markup { The odd-page footer }
        }

        \score {
            \new Staff {
                c'8
                d'8
                e'8
                f'8
            }
        }
        '''
        )



# -*- encoding: utf-8 -*-
from abjad import *


def test_agenttools_IterationAgent_by_logical_tie_01():
    r'''Yields successive logical ties.
    '''

    staff = Staff(scoretools.make_repeated_notes(4))
    tie = Tie()
    attach(tie, staff[:2])
    tie = Tie()
    attach(tie, staff[2:])

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8 ~
            c'8
            c'8 ~
            c'8
        }
        '''
        )

    logical_ties = list(iterate(staff).by_logical_tie(reverse=True))

    assert logical_ties[0] == selectiontools.LogicalTie((staff[2], staff[3]))
    assert logical_ties[1] == selectiontools.LogicalTie((staff[0], staff[1]))


def test_agenttools_IterationAgent_by_logical_tie_02():
    r'''Yields successive logical ties.
    '''

    staff = Staff(scoretools.make_repeated_notes(4))

    logical_ties = list(iterate(staff).by_logical_tie(reverse=True))

    assert logical_ties[0] == selectiontools.LogicalTie(staff[3])
    assert logical_ties[1] == selectiontools.LogicalTie(staff[2])
    assert logical_ties[2] == selectiontools.LogicalTie(staff[1])
    assert logical_ties[3] == selectiontools.LogicalTie(staff[0])


def test_agenttools_IterationAgent_by_logical_tie_03():
    r'''Yields successive logical ties.
    '''

    staff = Staff(scoretools.make_repeated_notes(4))
    tie = Tie()
    attach(tie, staff[:2])
    tie = Tie()
    attach(tie, staff[2:])

    logical_ties = list(iterate(staff).by_logical_tie())

    assert logical_ties[0] == selectiontools.LogicalTie((staff[0], staff[1]))
    assert logical_ties[1] == selectiontools.LogicalTie((staff[2], staff[3]))


def test_agenttools_IterationAgent_by_logical_tie_04():
    r'''Yields successive logical ties.
    '''

    staff = Staff("c'8 c'8 c'8 c'8")
    logical_ties = list(iterate(staff).by_logical_tie())

    assert logical_ties[0] == selectiontools.LogicalTie(staff[0])
    assert logical_ties[1] == selectiontools.LogicalTie(staff[1])
    assert logical_ties[2] == selectiontools.LogicalTie(staff[2])
    assert logical_ties[3] == selectiontools.LogicalTie(staff[3])


def test_agenttools_IterationAgent_by_logical_tie_05():

    staff = Staff(r"c'4 ~ \times 2/3 { c'16 d'8 } e'8 f'4 ~ f'16")

    logical_ties = iterate(staff).by_logical_tie(nontrivial=True, reverse=True)
    logical_ties = list(logical_ties)

    assert logical_ties[0].leaves == staff.select_leaves()[-2:]
    assert logical_ties[1].leaves == staff.select_leaves()[:2]


def test_agenttools_IterationAgent_by_logical_tie_06():

    staff = Staff(r"c'4 ~ \times 2/3 { c'16 d'8 } e'8 f'4 ~ f'16")

    logical_ties = list(iterate(staff).by_logical_tie(nontrivial=True))

    assert logical_ties[0].leaves == staff.select_leaves()[:2]
    assert logical_ties[1].leaves == staff.select_leaves()[-2:]


def test_agenttools_IterationAgent_by_logical_tie_07():

    staff = Staff('c ~ c r d ~ d r')

    logical_ties = iterate(staff).by_logical_tie(pitched=True, reverse=True)
    logical_ties = list(logical_ties)

    assert logical_ties[0] == selectiontools.LogicalTie(staff[3:5])
    assert logical_ties[1] == selectiontools.LogicalTie(staff[:2])


def test_agenttools_IterationAgent_by_logical_tie_08():

    staff = Staff('c ~ c r d ~ d r')

    logical_ties = list(iterate(staff).by_logical_tie(pitched=True))

    assert logical_ties[0] == selectiontools.LogicalTie(staff[:2])
    assert logical_ties[1] == selectiontools.LogicalTie(staff[3:5])



# -*- encoding: utf-8 -*-
from abjad import *


def test_spannertools_Beam_fracture_01():
    r'''This test shows that fracturing beyond the first leaf
    effectively does nothing except to replace an existing
    spanner with an identical new spanner.
    '''

    staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    beam = Beam()
    attach(beam, staff[:4])
    beam.fracture(0, direction=Left)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8 [
            d'8
            e'8
            f'8 ]
            g'8
            a'8
            b'8
            c''8
        }
        '''
        )

    assert inspect(staff).is_well_formed()


def test_spannertools_Beam_fracture_02():

    staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    beam = Beam()
    attach(beam, staff[:4])
    beam.fracture(1, direction=Left)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8 [ ]
            d'8 [
            e'8
            f'8 ]
            g'8
            a'8
            b'8
            c''8
        }
        '''
        )

    assert inspect(staff).is_well_formed()


def test_spannertools_Beam_fracture_03():
    r'''This test shows that fracurting beyond the last leaf
    effectively does nothing except to replace an existing
    spanner with an identical new spanner.
    '''

    staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    beam = Beam()
    attach(beam, staff[:4])
    beam.fracture(-1, direction=Right)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8 [
            d'8
            e'8
            f'8 ]
            g'8
            a'8
            b'8
            c''8
        }
        '''
        )

    assert inspect(staff).is_well_formed()


def test_spannertools_Beam_fracture_04():

    staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    beam = Beam()
    attach(beam, staff[:4])
    beam.fracture(1, direction=Right)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8 [
            d'8 ]
            e'8 [
            f'8 ]
            g'8
            a'8
            b'8
            c''8
        }
        '''
        )

    assert inspect(staff).is_well_formed()


def test_spannertools_Beam_fracture_05():
    r'''Fracture both sides of leaf.
    '''

    staff = Staff("c'8 cs'8 d'8 ef'8 e'8 f'8 fs'8 g'8")
    beam = Beam()
    attach(beam, staff[:5])
    beam.fracture(2, direction=None)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8 [
            cs'8 ]
            d'8 [ ]
            ef'8 [
            e'8 ]
            f'8
            fs'8
            g'8
        }
        '''
        )

    assert inspect(staff).is_well_formed()


def test_spannertools_Beam_fracture_06():
    r'''Fracture both sides of first leaf in spanner.
    '''

    staff = Staff("c'8 cs'8 d'8 ef'8 e'8 f'8 fs'8 g'8")
    beam = Beam()
    attach(beam, staff[:5])
    beam.fracture(0, direction=None)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8 [ ]
            cs'8 [
            d'8
            ef'8
            e'8 ]
            f'8
            fs'8
            g'8
        }
        '''
        )

    assert inspect(staff).is_well_formed()


def test_spannertools_Beam_fracture_07():
    r'''Fracture both sides of last leaf in spanner.
    '''

    staff = Staff("c'8 cs'8 d'8 ef'8 e'8 f'8 fs'8 g'8")
    beam = Beam()
    attach(beam, staff[:5])
    beam.fracture(4, direction=None)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8 [
            cs'8
            d'8
            ef'8 ]
            e'8 [ ]
            f'8
            fs'8
            g'8
        }
        '''
        )

    assert inspect(staff).is_well_formed()


def test_spannertools_Beam_fracture_08():
    r'''Fracture both sides of leaf with negative index.
    '''

    staff = Staff("c'8 cs'8 d'8 ef'8 e'8 f'8 fs'8 g'8")
    beam = Beam()
    attach(beam, staff[:5])
    beam.fracture(-1, direction=None)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8 [
            cs'8
            d'8
            ef'8 ]
            e'8 [ ]
            f'8
            fs'8
            g'8
        }
        '''
        )

    assert inspect(staff).is_well_formed()



# -*- encoding: utf-8 -*-
import fractions


def fraction_to_proper_fraction(rational):
    '''Changes `rational` to proper fraction.

    ::

        >>> mathtools.fraction_to_proper_fraction(Fraction(116, 8))
        (14, Fraction(1, 2))

    Returns pair.
    '''

    if not isinstance(rational, fractions.Fraction):
        raise TypeError

    quotient = int(rational)
    residue = rational - quotient

    return quotient, residue



# -*- encoding: utf-8 -*-
from abjad.tools.mathtools.Infinity import Infinity


class NegativeInfinity(Infinity):
    r'''Object-oriented negative infinity.

    All numbers compare greater than negative infinity:

    ::

        >>> NegativeInfinity < -9999999
        True

    Negative infinity compares equal to itself:

    ::

        >>> NegativeInfinity == NegativeInfinity
        True

    Negative infinity compares less than infinity:

    ::

        >>> NegativeInfinity < Infinity
        True

    Negative infinity is initialize at start-up and is available in the 
    global Abjad namespace.
    '''

    ### INITIALIZER ###

    def __init__(self):
        self._value = float('-infinity')



# -*- encoding: utf-8 -*-
from abjad.tools.datastructuretools.TypedList import TypedList


class TempoInventory(TypedList):
    r'''An ordered list of tempo indications.

    ::

        >>> inventory = indicatortools.TempoInventory([
        ...     ('Andante', Duration(1, 8), 72),
        ...     ('Allegro', Duration(1, 8), 84),
        ...     ])

    ::

        >>> for tempo in inventory:
        ...     tempo
        ...
        Tempo('Andante', Duration(1, 8), 72)
        Tempo('Allegro', Duration(1, 8), 84)

    Tempo inventories implement list interface and are mutable.
    '''

    ### PRIVATE PROPERTIES ###

    @property
    def _item_callable(self):
        from abjad.tools import indicatortools
        return indicatortools.Tempo



# -*- encoding: utf-8 -*-
from abjad.tools import sequencetools
from abjad.tools.topleveltools import iterate


def iterate_named_pitch_pairs_in_expr(expr):
    r'''Iterates left-to-right, top-to-bottom named pitch pairs in `expr`.

    ::

        >>> score = Score([])
        >>> notes = [Note("c'8"), Note("d'8"), Note("e'8"), Note("f'8"), Note("g'4")]
        >>> score.append(Staff(notes))
        >>> notes = [Note(x, (1, 4)) for x in [-12, -15, -17]]
        >>> score.append(Staff(notes))
        >>> clef = Clef('bass')
        >>> attach(clef, score[1])
        >>> show(score) # doctest: +SKIP

    ..  doctest::

        >>> print format(score)
        \new Score <<
            \new Staff {
                c'8
                d'8
                e'8
                f'8
                g'4
            }
            \new Staff {
                \clef "bass"
                c4
                a,4
                g,4
            }
        >>

    ::

        >>> for pair in pitchtools.iterate_named_pitch_pairs_in_expr(score):
        ...     pair
        ...
        (NamedPitch("c'"), NamedPitch('c'))
        (NamedPitch("c'"), NamedPitch("d'"))
        (NamedPitch('c'), NamedPitch("d'"))
        (NamedPitch("d'"), NamedPitch("e'"))
        (NamedPitch("d'"), NamedPitch('a,'))
        (NamedPitch('c'), NamedPitch("e'"))
        (NamedPitch('c'), NamedPitch('a,'))
        (NamedPitch("e'"), NamedPitch('a,'))
        (NamedPitch("e'"), NamedPitch("f'"))
        (NamedPitch('a,'), NamedPitch("f'"))
        (NamedPitch("f'"), NamedPitch("g'"))
        (NamedPitch("f'"), NamedPitch('g,'))
        (NamedPitch('a,'), NamedPitch("g'"))
        (NamedPitch('a,'), NamedPitch('g,'))
        (NamedPitch("g'"), NamedPitch('g,'))

    Chords are handled correctly. ::

        >>> chord_1 = Chord([0, 2, 4], (1, 4))
        >>> chord_2 = Chord([17, 19], (1, 4))
        >>> staff = Staff([chord_1, chord_2])

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            <c' d' e'>4
            <f'' g''>4
        }

    ::

        >>> for pair in pitchtools.iterate_named_pitch_pairs_in_expr(staff):
        ...   print pair
        (NamedPitch("c'"), NamedPitch("d'"))
        (NamedPitch("c'"), NamedPitch("e'"))
        (NamedPitch("d'"), NamedPitch("e'"))
        (NamedPitch("c'"), NamedPitch("f''"))
        (NamedPitch("c'"), NamedPitch("g''"))
        (NamedPitch("d'"), NamedPitch("f''"))
        (NamedPitch("d'"), NamedPitch("g''"))
        (NamedPitch("e'"), NamedPitch("f''"))
        (NamedPitch("e'"), NamedPitch("g''"))
        (NamedPitch("f''"), NamedPitch("g''"))

    Returns generator.
    '''
    from abjad.tools import pitchtools

    for leaf_pair in iterate(expr).by_leaf_pair():
        leaf_pair_list = list(leaf_pair)
        # iterate chord pitches if first leaf is chord
        for pair in pitchtools.list_unordered_named_pitch_pairs_in_expr(leaf_pair_list[0]):
            yield pair
        if isinstance(leaf_pair, set):
            for pair in pitchtools.list_unordered_named_pitch_pairs_in_expr(leaf_pair):
                yield pair
        elif isinstance(leaf_pair, tuple):
            for pair in pitchtools.list_ordered_named_pitch_pairs_from_expr_1_to_expr_2(*leaf_pair):
                yield pair
        else:
            message = 'leaf pair must be set or tuple.'
            raise TypeError(message)
        # iterate chord pitches if last leaf is chord
        for pair in pitchtools.list_unordered_named_pitch_pairs_in_expr(leaf_pair_list[1]):
            yield pair



#[ -*- encoding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import sequencetools
from abjad.tools.spannertools.ComplexBeam import ComplexBeam


class DuratedComplexBeam(ComplexBeam):
    r'''A durated complex beam spanner.

    ::

        >>> staff = Staff("c'16 d'16 e'16 f'16")
        >>> show(staff) # doctest: +SKIP

    ::

        >>> durations = [Duration(1, 8), Duration(1, 8)]
        >>> beam = spannertools.DuratedComplexBeam(
        ...     durations=durations, 
        ...     span=1,
        ...     )
        >>> attach(beam, staff[:])
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            \set stemLeftBeamCount = #0
            \set stemRightBeamCount = #2
            c'16 [
            \set stemLeftBeamCount = #2
            \set stemRightBeamCount = #1
            d'16
            \set stemLeftBeamCount = #1
            \set stemRightBeamCount = #2
            e'16
            \set stemLeftBeamCount = #2
            \set stemRightBeamCount = #0
            f'16 ]
        }

    Beam all beamable leaves in spanner explicitly.

    Group leaves in spanner according to `durations`.

    Span leaves between duration groups according to `span`.

    Returns durated complex beam spanner.
    '''

    def __init__(self, 
        components=None, 
        durations=None, 
        span=1, 
        lone=False, 
        direction=None,
        overrides=None,
        ):
        ComplexBeam.__init__(
            self, 
            components=components, 
            direction=direction,
            overrides=overrides,
            )
        self.durations = durations
        self.span = span
        self.lone = lone

    ### PRIVATE PROPERTIES ###

    @property
    def _span_points(self):
        result = []
        if self.durations is not None:
            result.append(self.durations[0])
            for d in self.durations[1:]:
                result.append(result[-1] + d)
        return result

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        ComplexBeam._copy_keyword_args(self, new)
        new.durations = self.durations[:]
        new.span = self.span

    def _format_before_leaf(self, leaf):
        result = []
        #if leaf.beam.beamable:
        if self.is_beamable_component(leaf):
            if self._is_exterior_leaf(leaf):
                left, right = self._get_left_right_for_exterior_leaf(leaf)
            # just right of span gap
            elif self._duration_offset_in_me(leaf) in self._span_points and \
                not (self._duration_offset_in_me(leaf) + leaf._get_duration() in \
                self._span_points):
                assert isinstance(self.span, int)
                left = self.span
                #right = leaf._get_duration()._flags
                right = leaf.written_duration.flag_count
            # just left of span gap
            elif self._duration_offset_in_me(leaf) + leaf._get_duration() in \
                self._span_points and \
                not self._duration_offset_in_me(leaf) in self._span_points:
                assert isinstance(self.span, int)
                #left = leaf._get_duration()._flags
                left = leaf.written_duration.flag_count
                right = self.span
            else:
                left, right = self._get_left_right_for_interior_leaf(leaf)
            if left is not None:
                result.append(r'\set stemLeftBeamCount = #%s' % left)
            if right is not None:
                result.append(r'\set stemRightBeamCount = #%s' % right)
        return result

    def _fracture_left(self, i):
        self, left, right = ComplexBeam._fracture_left(self, i)
        weights = [left.get_duration(), right.get_duration()]
        assert sum(self.durations) == sum(weights)
        split_durations = sequencetools.split_sequence_by_weights(
            self.durations, weights, cyclic=False, overhang=False)
        left_durations, right_durations = split_durations
        left._durations = left_durations
        right._durations = right_durations
        return self, left, right

    def _fracture_right(self, i):
        self, left, right = ComplexBeam._fracture_right(self, i)
        weights = [left.get_duration(), right.get_duration()]
        assert sum(self.durations) == sum(weights)
        split_durations = sequencetools.split_sequence_by_weights(
            self.durations, weights, cyclic=False, overhang=False)
        left_durations, right_durations = split_durations
        left._durations = left_durations
        right._durations = right_durations
        return self, left, right

    def _reverse_components(self):
        ComplexBeam._reverse_components(self)
        self._durations.reverse()

    ### PUBLIC PROPERTIES ###

    @property
    def durations(self):
        r'''Get spanner leaf group durations:

        ::

            >>> staff = Staff("c'16 d'16 e'16 f'16")
            >>> durations = [Duration(1, 8), Duration(1, 8)]
            >>> beam = spannertools.DuratedComplexBeam(
            ...     durations=durations,
            ...     )
            >>> attach(beam, staff[:])
            >>> beam.durations
            [Duration(1, 8), Duration(1, 8)]

        Set spanner leaf group durations:

        ::

            >>> staff = Staff("c'16 d'16 e'16 f'16")
            >>> durations = [Duration(1, 8), Duration(1, 8)]
            >>> beam = spannertools.DuratedComplexBeam(
            ...     durations=durations,
            ...     )
            >>> attach(beam, staff[:])
            >>> beam.durations = [Duration(1, 4)]
            >>> beam.durations
            [Duration(1, 4)]

        Set iterable.
        '''
        return self._durations

    @durations.setter
    def durations(self, arg):
        if arg is None:
            self._durations = None
        elif isinstance(arg, list):
            for i, d in enumerate(arg):
                if isinstance(d, tuple):
                    arg[i] = durationtools.Duration(*d)
                else:
                    arg[i] = durationtools.Duration(d)
            self._durations = arg
        else:
            message = 'durations must be list of durations or none.'
            raise ValueError(message)

    @property
    def span(self):
        r'''Get top-level beam count:

        ::

            >>> staff = Staff("c'16 d'16 e'16 f'16")
            >>> durations = [Duration(1, 8), Duration(1, 8)]
            >>> beam = spannertools.DuratedComplexBeam(
            ...     durations=durations, 
            ...     span=1,
            ...     )
            >>> attach(beam, staff[:])
            >>> beam.span
            1

        Set top-level beam count:

        ::

            >>> staff = Staff("c'16 d'16 e'16 f'16")
            >>> durations = [Duration(1, 8), Duration(1, 8)]
            >>> beam = spannertools.DuratedComplexBeam(
            ...     durations=durations, 
            ...     span=1,
            ...     )
            >>> attach(beam, staff[:])
            >>> beam.span = 2
            >>> beam.span
            2

        Set nonnegative integer.
        '''
        return self._span

    @span.setter
    def span(self, arg):
        assert isinstance(arg, (int, type(None)))
        self._span = arg



# -*- encoding: utf-8 -*-
from abjad.tools.stringtools.is_snake_case_string \
	import underscore_delimited_lowercase_regex_body
import re


underscore_delimited_lowercase_file_name_regex_body = """
    %s
    (\.[a-z,0-9]+)?
    """ % underscore_delimited_lowercase_regex_body

underscore_delimited_lowercase_file_name_regex = re.compile('^%s$' %
    underscore_delimited_lowercase_file_name_regex_body, re.VERBOSE)

def is_snake_case_file_name(expr):
    r'''True when `expr` is a string and is underscore-delimited lowercase file name with extension:

    ::

        >>> stringtools.is_snake_case_file_name('foo_bar')
        True

    Otherwise false:

    ::

        >>> stringtools.is_snake_case_file_name('foo.bar.blah')
        False

    Returns boolean.
    '''

    if not isinstance(expr, str):
        return False

    if expr == '':
        return True

    return bool(underscore_delimited_lowercase_file_name_regex.match(expr))



# -*- encoding: utf-8 -*-
from abjad import *


def test_quantizationtools_Quantizer___call___01():

    milliseconds = [1500, -1000, 1000, 1000, -1000, 1000, -1000, 500]
    sequence = quantizationtools.QEventSequence.from_millisecond_durations(milliseconds)
    attack_point_optimizer = quantizationtools.NullAttackPointOptimizer()
    quantizer = quantizationtools.Quantizer()

    result = quantizer(sequence, attack_point_optimizer=attack_point_optimizer)

    assert isinstance(result, Voice)
    assert inspect(result).get_duration() == 2

    score = Score([Staff([result])])

    assert systemtools.TestManager.compare(
        score,
        r'''
        \new Score <<
            \new Staff {
                \new Voice {
                    {
                        \tempo 4=60
                        \time 4/4
                        c'4 ~
                        c'8
                        r8 ~
                        r8
                        c'8 ~
                        c'8
                        c'8 ~
                    }
                    {
                        c'8
                        r8 ~
                        r8
                        c'8 ~
                        c'8
                        r8 ~
                        r8
                        c'8
                    }
                }
            }
        >>
        '''
        )


def test_quantizationtools_Quantizer___call___02():

    milliseconds = [250, 1000, 1000, 1000, 750]
    sequence = quantizationtools.QEventSequence.from_millisecond_durations(milliseconds)
    attack_point_optimizer = quantizationtools.NullAttackPointOptimizer()
    quantizer = quantizationtools.Quantizer()

    result = quantizer(sequence, attack_point_optimizer=attack_point_optimizer)

    assert systemtools.TestManager.compare(
        result,
        r'''
        \new Voice {
            {
                %%% \tempo 4=60 %%%
                \time 4/4
                c'16
                c'16 ~
                c'8 ~
                c'16
                c'16 ~
                c'8 ~
                c'16
                c'16 ~
                c'8 ~
                c'16
                c'16 ~
                c'8
            }
        }
        '''
        )


def test_quantizationtools_Quantizer___call___03():

    q_schema = quantizationtools.BeatwiseQSchema(
        {'search_tree': quantizationtools.UnweightedSearchTree({2: None})},
        {'search_tree': quantizationtools.UnweightedSearchTree({3: None})},
        {'search_tree': quantizationtools.UnweightedSearchTree({5: None})},
        {'search_tree': quantizationtools.UnweightedSearchTree({7: None})},
        )
    milliseconds = [250, 250, 250, 250] * 4
    sequence = quantizationtools.QEventSequence.from_millisecond_durations(milliseconds)
    attack_point_optimizer = quantizationtools.NullAttackPointOptimizer()
    quantizer = quantizationtools.Quantizer()

    result = quantizer(sequence, q_schema=q_schema, attack_point_optimizer=attack_point_optimizer)

    assert systemtools.TestManager.compare(
        result,
        r'''
        \new Voice {
            \grace {
                c'16
            }
            %%% \tempo 4=60 %%%
            c'8
            \grace {
                c'16
            }
            c'8
            \times 2/3 {
                c'8
                \grace {
                    c'16
                }
                c'8
                c'8
            }
            \times 4/5 {
                c'16
                c'16
                c'16 ~
                c'16
                c'16
            }
            \times 4/7 {
                c'16 ~
                c'16
                c'16
                c'16 ~
                c'16
                c'16 ~
                c'16
            }
        }
        '''
        )



# -*- encoding: utf-8 -*-
from abjad import *


def test_rhythmmakertools_TaleaRhythmMaker_tie_split_notes_01():

    maker = rhythmmakertools.TaleaRhythmMaker([5], 16, tie_split_notes=True)
    divisions = [(2, 8), (2, 8), (2, 8), (2, 8)]
    music = maker(divisions)

    music = sequencetools.flatten_sequence(music)
    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    measures = scoretools.replace_contents_of_measures_in_expr(staff, music)

    r'''
    \new Staff {
        {
            \time 2/8
            c'4 ~
        }
        {
            c'16
            c'8. ~
        }
        {
            c'8
            c'8 ~
        }
        {
            c'8.
            c'16
        }
    }
    '''

    assert inspect(staff).is_well_formed()
    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 2/8
                c'4 ~
            }
            {
                c'16
                c'8. ~
            }
            {
                c'8
                c'8 ~
            }
            {
                c'8.
                c'16
            }
        }
        '''
        )


def test_rhythmmakertools_TaleaRhythmMaker_tie_split_notes_02():

    maker = rhythmmakertools.TaleaRhythmMaker([5], 16, tie_split_notes=True)
    divisions = [(3, 16), (5, 8), (4, 8), (7, 16)]
    music = maker(divisions)

    music = sequencetools.flatten_sequence(music)
    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    measures = scoretools.replace_contents_of_measures_in_expr(staff, music)

    r'''
    \new Staff {
        {
            \time 3/16
            c'8. ~
        }
        {
            \time 5/8
            c'8
            c'4 ~
            c'16
            c'8. ~
        }
        {
            \time 4/8
            c'8
            c'4 ~
            c'16
            c'16 ~
        }
        {
            \time 7/16
            c'4
            c'8.
        }
    }
    '''

    assert inspect(staff).is_well_formed()
    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 3/16
                c'8. ~
            }
            {
                \time 5/8
                c'8
                c'4 ~
                c'16
                c'8. ~
            }
            {
                \time 4/8
                c'8
                c'4 ~
                c'16
                c'16 ~
            }
            {
                \time 7/16
                c'4
                c'8.
            }
        }
        '''
        )



# -*- encoding: utf-8 -*-
from abjad.tools import mathtools
import itertools


def sum_consecutive_sequence_elements_by_sign(sequence, sign=(-1, 0, 1)):
    '''Sum consecutive `sequence` elements by `sign`:

    ::

        >>> sequence = [0, 0, -1, -1, 2, 3, -5, 1, 2, 5, -5, -6]

    ::

        >>> sequencetools.sum_consecutive_sequence_elements_by_sign(sequence)
        [0, -2, 5, -5, 8, -11]

    ::

        >>> sequencetools.sum_consecutive_sequence_elements_by_sign(sequence, sign=[-1])
        [0, 0, -2, 2, 3, -5, 1, 2, 5, -11]

    ::

        >>> sequencetools.sum_consecutive_sequence_elements_by_sign(sequence, sign=[0])
        [0, -1, -1, 2, 3, -5, 1, 2, 5, -5, -6]

    ::

        >>> sequencetools.sum_consecutive_sequence_elements_by_sign(sequence, sign=[1])
        [0, 0, -1, -1, 5, -5, 8, -5, -6]

    ::

        >>> sequencetools.sum_consecutive_sequence_elements_by_sign(sequence, sign=[-1, 0])
        [0, -2, 2, 3, -5, 1, 2, 5, -11]

    ::

        >>> sequencetools.sum_consecutive_sequence_elements_by_sign(sequence, sign=[-1, 1])
        [0, 0, -2, 5, -5, 8, -11]

    ::

        >>> sequencetools.sum_consecutive_sequence_elements_by_sign(sequence, sign=[0, 1])
        [0, -1, -1, 5, -5, 8, -5, -6]

    ::

        >>> sequencetools.sum_consecutive_sequence_elements_by_sign(sequence, sign=[-1, 0, 1])
        [0, -2, 5, -5, 8, -11]

    When ``-1`` in `sign`, sum consecutive negative elements.

    When ``0`` in `sign`, sum consecutive ``0`` elements.

    When ``1`` in `sign`, sum consecutive positive elements.

    Returns list.
    '''

    result = []

    generator = itertools.groupby(sequence, mathtools.sign)
    for current_sign, group in generator:
        if current_sign in sign:
            result.append(sum(group))
        else:
            for x in group:
                result.append(x)

    return result



# -*- encoding: utf-8 -*-
import inspect
from abjad.tools.documentationtools.Documenter import Documenter


class ClassDocumenter(Documenter):
    r'''Generates an ReST API entry for a given class.

    ::

        >>> cls = documentationtools.ClassDocumenter
        >>> documenter = documentationtools.ClassDocumenter(cls)
        >>> restructured_text = documenter()
        >>> print restructured_text
        documentationtools.ClassDocumenter
        ==================================
        <BLANKLINE>
        .. abjad-lineage:: abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter
        <BLANKLINE>
        .. autoclass:: abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter
        <BLANKLINE>
        .. only:: html
        <BLANKLINE>
           Attribute summary
           -----------------
        <BLANKLINE>
           .. autosummary::
        <BLANKLINE>
              ~abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.class_methods
              ~abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.data
              ~abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.inherited_attributes
              ~abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.is_abstract
              ~abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.methods
              ~abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.module_name
              ~abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.object_
              ~abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.prefix
              ~abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.readonly_properties
              ~abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.readwrite_properties
              ~abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.special_methods
              ~abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.static_methods
              ~abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.write
              ~abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.__call__
              ~abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.__eq__
              ~abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.__makenew__
              ~abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.__ne__
              ~abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.__repr__
        <BLANKLINE>
        Bases
        -----
        <BLANKLINE>
        - :py:class:`documentationtools.Documenter <abjad.tools.documentationtools.Documenter.Documenter>`
        <BLANKLINE>
        - :py:class:`abctools.AbjadObject <abjad.tools.abctools.AbjadObject.AbjadObject>`
        <BLANKLINE>
        - :py:class:`__builtin__.object <object>`
        <BLANKLINE>
        Read-only properties
        --------------------
        <BLANKLINE>
        .. autoattribute:: abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.class_methods
           :noindex:
        <BLANKLINE>
        .. autoattribute:: abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.data
           :noindex:
        <BLANKLINE>
        .. autoattribute:: abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.inherited_attributes
           :noindex:
        <BLANKLINE>
        .. autoattribute:: abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.is_abstract
           :noindex:
        <BLANKLINE>
        .. autoattribute:: abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.methods
           :noindex:
        <BLANKLINE>
        .. autoattribute:: abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.module_name
           :noindex:
        <BLANKLINE>
        .. autoattribute:: abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.object_
           :noindex:
        <BLANKLINE>
        .. autoattribute:: abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.prefix
           :noindex:
        <BLANKLINE>
        .. autoattribute:: abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.readonly_properties
           :noindex:
        <BLANKLINE>
        .. autoattribute:: abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.readwrite_properties
           :noindex:
        <BLANKLINE>
        .. autoattribute:: abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.special_methods
           :noindex:
        <BLANKLINE>
        .. autoattribute:: abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.static_methods
           :noindex:
        <BLANKLINE>
        Static methods
        --------------
        <BLANKLINE>
        .. automethod:: abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.write
           :noindex:
        <BLANKLINE>
        Special methods
        ---------------
        <BLANKLINE>
        .. automethod:: abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.__call__
           :noindex:
        <BLANKLINE>
        .. automethod:: abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.__eq__
           :noindex:
        <BLANKLINE>
        .. automethod:: abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.__makenew__
           :noindex:
        <BLANKLINE>
        .. automethod:: abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.__ne__
           :noindex:
        <BLANKLINE>
        .. automethod:: abjad.tools.documentationtools.ClassDocumenter.ClassDocumenter.__repr__
           :noindex:
        <BLANKLINE>

    '''

    ### CLASS VARIABLES ###

    _ignored_special_methods = (
        '__getattribute__',
        '__getnewargs__',
        '__getstate__',
        '__init__',
        '__reduce__',
        '__reduce_ex__',
        '__setstate__',
        '__sizeof__',
        '__subclasshook__',
        'fromkeys',
        'pipe_cloexec',
        )

    ### INITIALIZER ###

    def __init__(self, object_=None, prefix='abjad.tools.'):
        if object_ is None:
            object_ = type(None)
        assert isinstance(object_, type)
        Documenter.__init__(self, object_, prefix)

        class_methods = []
        data = []
        inherited_attributes = []
        methods = []
        readonly_properties = []
        readwrite_properties = []
        special_methods = []
        static_methods = []

        attrs = inspect.classify_class_attrs(self.object_)
        for attr in attrs:
            if attr.defining_class is object:
                continue
            if self._attribute_is_inherited(attr):
                inherited_attributes.append(attr)
            if attr.kind == 'method':
                if attr.name not in self._ignored_special_methods:
                    if attr.name.startswith('__'):
                        special_methods.append(attr)
                    elif not attr.name.startswith('_'):
                        methods.append(attr)
            elif attr.kind == 'class method':
                if attr.name not in self._ignored_special_methods:
                    if attr.name.startswith('__'):
                        special_methods.append(attr)
                    elif not attr.name.startswith('_'):
                        class_methods.append(attr)
            elif attr.kind == 'static method':
                if attr.name not in self._ignored_special_methods:
                    if attr.name.startswith('__'):
                        special_methods.append(attr)
                    elif not attr.name.startswith('_'):
                        static_methods.append(attr)
            elif attr.kind == 'property' and not attr.name.startswith('_'):
                if attr.object.fset is None:
                    readonly_properties.append(attr)
                else:
                    readwrite_properties.append(attr)
            elif attr.kind == 'data' and not attr.name.startswith('_') \
                and attr.name not in getattr(self.object_, '__slots__', ()):
                data.append(attr)

        self._class_methods = tuple(sorted(class_methods))
        self._data = tuple(sorted(data))
        self._inherited_attributes = tuple(sorted(inherited_attributes))
        self._methods = tuple(sorted(methods))
        self._readonly_properties = tuple(sorted(readonly_properties))
        self._readwrite_properties = tuple(sorted(readwrite_properties))
        self._special_methods = tuple(sorted(special_methods))
        self._static_methods = tuple(sorted(static_methods))

    ### SPECIAL METHODS ###

    def __call__(self):
        r'''Calls class documenter.

        Generates documentation.

        Returns string.
        '''
        from abjad.tools import documentationtools

        stripped_class_name = self._shrink_module_name(self.object_.__module__)
        parts = self.object_.__module__.split('.')
        tools_package_path = '.'.join(parts[:3])
        tools_package_name, sep, class_name = stripped_class_name.partition('.')
        banner = '{}.{}'.format(tools_package_name, class_name)
#        banner = ':py:mod:`{} <{}>`.{}'.format(
#            tools_package_name,
#            tools_package_path,
#            class_name,
#            )
        document = documentationtools.ReSTDocument()
        document.append(documentationtools.ReSTHeading(
            level=2,
            text=banner,
            ))
        document.append(documentationtools.ReSTLineageDirective(
            argument=self.module_name,
            ))
        document.append(documentationtools.ReSTAutodocDirective(
            argument=self.module_name,
            directive='autoclass',
            options={
                #'noindex': True,
                },
            ))
        document.extend(self._build_attributes_autosummary())
        document.extend(self._build_bases_section())
#        document.extend(self._build_attribute_section(
#            self.data,
#            'Class variables',
#            'autodata',
#            ))
        document.extend(self._build_attribute_section(
            self.readonly_properties,
            'Read-only properties',
            'autoattribute',
            ))
        document.extend(self._build_attribute_section(
            self.readwrite_properties,
            'Read/write properties',
            'autoattribute',
            ))
        document.extend(self._build_attribute_section(
            self.methods,
            'Methods',
            'automethod',
            ))
        document.extend(self._build_attribute_section(
            self.class_methods,
            'Class methods',
            'automethod',
            ))
        document.extend(self._build_attribute_section(
            self.static_methods,
            'Static methods',
            'automethod',
            ))
        document.extend(self._build_attribute_section(
            self.special_methods,
            'Special methods',
            'automethod',
            ))
        return document.rest_format

    ### PRIVATE PROPERTIES ###

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        positional_argument_values = (
            self.object_
            )
        return systemtools.StorageFormatSpecification(
            self,
            keyword_argument_names=(),
            positional_argument_values=positional_argument_values,
            )

    ### PRIVATE METHODS ###

    def _attribute_is_inherited(self, attr):
        if attr.defining_class is not self.object_:
            return True
        return False

    def _build_attribute_section(self, attrs, title, directive):
        from abjad.tools import documentationtools
        result = []
        if attrs:
            result.append(documentationtools.ReSTHeading(
                level=3,
                text=title,
                ))
            for attr in attrs:
                autodoc = documentationtools.ReSTAutodocDirective(
                    argument='{}.{}'.format(
                        self.module_name,
                        attr.name,
                        ),
                    directive=directive,
                    options={
                        'noindex': True,
                        },
                    )
                result.append(autodoc)
        return result

    def _build_attributes_autosummary(self):
        from abjad.tools import documentationtools
        pieces = []
        attributes = []
        attributes.extend(self.readonly_properties)
        attributes.extend(self.readwrite_properties)
        attributes.extend(self.methods)
        attributes.extend(self.class_methods)
        attributes.extend(self.static_methods)
        attributes.sort(key=lambda x: x.name)
        attributes.extend(self.special_methods)
        autosummary = documentationtools.ReSTAutosummaryDirective()
        for attribute in attributes:
            autosummary.append('~{}.{}'.format(
                self.module_name,
                attribute.name,
                ))
        html_only = documentationtools.ReSTOnlyDirective(argument='html')
        html_only.append(documentationtools.ReSTHeading(
            level=3,
            text='Attribute summary',
            ))
        html_only.append(autosummary)
        pieces.append(html_only)
        return pieces

    def _build_bases_section(self):
        from abjad.tools import documentationtools
        pieces = []
        pieces.append(documentationtools.ReSTHeading(
            level=3,
            text='Bases',
            ))
        mro = inspect.getmro(self.object_)[1:]
        for cls in mro:
            packagesystem_path = '.'.join((cls.__module__, cls.__name__))
            stripped_packagesystem_path = self._shrink_module_name(
                packagesystem_path)
            if packagesystem_path.startswith('__builtin__'):
                packagesystem_path = \
                    packagesystem_path.partition('__builtin__.')[-1]
            text = '- :py:class:`{} <{}>`'.format(
                stripped_packagesystem_path,
                packagesystem_path,
                )
            paragraph = documentationtools.ReSTParagraph(
                text=text,
                wrap=False,
                )
            pieces.append(paragraph)
        return pieces

    ### PUBLIC PROPERTIES ###

    @property
    def class_methods(self):
        r'''Class methods.
        '''
        return self._class_methods

    @property
    def data(self):
        r'''Data.
        '''
        return self._data

    @property
    def inherited_attributes(self):
        r'''Inherited attributes.
        '''
        return self._inherited_attributes

    @property
    def is_abstract(self):
        r'''True when class is abstract. Otherwise false.

        Returns boolean.
        '''
        return inspect.isabstract(self.object_)

    @property
    def methods(self):
        r'''Methods of class.
        '''
        return self._methods

    @property
    def readonly_properties(self):
        r'''Read-only properties of class.
        '''
        return self._readonly_properties

    @property
    def readwrite_properties(self):
        r'''The read / write properties of class.
        '''
        return self._readwrite_properties

    @property
    def special_methods(self):
        r'''Special methods of class.
        '''
        return self._special_methods

    @property
    def static_methods(self):
        r'''Static methods of class.
        '''
        return self._static_methods



# -*- encoding: utf-8 -*-
from abjad.tools import scoretools
from abjad.tools import scoretools
from abjad.tools import markuptools
from abjad.tools import scoretools
from abjad.tools import pitchtools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import iterate


def label_leaves_in_expr_with_numbered_intervals(expr, markup_direction=Up):
    r"""Label leaves in `expr` with numbered intervals:

    ::

        >>> notes = scoretools.make_notes(
        ...     [0, 25, 11, -4, -14, -13, 9, 10, 6, 5],
        ...     [Duration(1, 8)],
        ...     )
        >>> staff = Staff(notes)
        >>> labeltools.label_leaves_in_expr_with_numbered_intervals(staff)

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            c'8 ^ \markup { +25 }
            cs'''8 ^ \markup { -14 }
            b'8 ^ \markup { -15 }
            af8 ^ \markup { -10 }
            bf,8 ^ \markup { +1 }
            b,8 ^ \markup { +22 }
            a'8 ^ \markup { +1 }
            bf'8 ^ \markup { -4 }
            fs'8 ^ \markup { -1 }
            f'8
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Returns none.
    """

    for note in iterate(expr).by_class(scoretools.Note):
        logical_voice_iterator = iterate(note).by_logical_voice_from_component(
            scoretools.Leaf,
            )
        try:
            logical_voice_iterator.next()
            next_leaf = logical_voice_iterator.next()
            if isinstance(next_leaf, scoretools.Note):
                mci = pitchtools.NumberedInterval.from_pitch_carriers(
                    note, next_leaf)
                markup = markuptools.Markup(mci, markup_direction)
                attach(markup, note)
        except StopIteration:
            pass



# -*- encoding: utf-8 -*-
from abjad import *


def test_datastructuretools_CyclicMatrix___init___01():
    r'''Initializes from rows.
    '''

    cyclic_matrix = datastructuretools.CyclicMatrix((
        (0, 1, 2, 3), 
        (10, 11, 12, 13), 
        (20, 21, 22, 23),
        ))

    assert cyclic_matrix.rows == \
        ((0, 1, 2, 3), (10, 11, 12, 13), (20, 21, 22, 23))
    assert cyclic_matrix.columns == \
        ((0, 10, 20), (1, 11, 21), (2, 12, 22), (3, 13, 23))


def test_datastructuretools_CyclicMatrix___init___02():
    r'''Initializes from columns.
    '''

    cyclic_matrix = datastructuretools.CyclicMatrix(columns=(
        (0, 10, 20), 
        (1, 11, 21), 
        (2, 12, 22), 
        (3, 13, 23),
        ))

    assert cyclic_matrix.rows == \
        ((0, 1, 2, 3), (10, 11, 12, 13), (20, 21, 22, 23))
    assert cyclic_matrix.columns == \
        ((0, 10, 20), (1, 11, 21), (2, 12, 22), (3, 13, 23))


def test_datastructuretools_CyclicMatrix___init___03():
    r'''Initializes from rows of differing lengths.
    '''

    cyclic_matrix = datastructuretools.CyclicMatrix((
        (0, 1, 2, 3), 
        (10, 11, 12, 13), 
        (20, 21),
        ))

    assert cyclic_matrix.rows == ((0, 1, 2, 3), (10, 11, 12, 13), (20, 21))


def test_datastructuretools_CyclicMatrix___init___04():
    r'''Initializes from columns of differing lengths.
    '''

    cyclic_matrix = datastructuretools.CyclicMatrix(columns=(
        (0, 10, 20), 
        (1, 11, 21), 
        (2, 12), 
        (3, 13),
        ))

    assert cyclic_matrix.columns == \
        ((0, 10, 20), (1, 11, 21), (2, 12), (3, 13))



# -*- encoding: utf-8 -*-
from abjad import *


def test_scoretools_fill_measures_in_expr_with_time_signature_denominator_notes_01():
    r'''Populate non-power-of-two measure with time signature denominator notes.
    '''

    measure = Measure((5, 18), [])
    scoretools.fill_measures_in_expr_with_time_signature_denominator_notes(measure)

    r'''
    {
        \time 5/18
        \scaleDurations #'(8 . 9) {
            c'16
            c'16
            c'16
            c'16
            c'16
        }
    }
    '''

    assert inspect(measure).is_well_formed()
    assert systemtools.TestManager.compare(
        measure,
        r'''
        {
            \time 5/18
            \scaleDurations #'(8 . 9) {
                c'16
                c'16
                c'16
                c'16
                c'16
            }
        }
        '''
        )



# -*- encoding: utf-8 -*-
from abjad import *
from abjad.tools import layouttools


def test_layouttools_SpacingIndication___eq___01():
    r'''Spacing indications compare equal when
    normalized spacing durations compare equal.
    '''

    tempo_indication = Tempo(Duration(1, 8), 38)
    spacing_indication_1 = layouttools.SpacingIndication(
        tempo_indication, Duration(1, 68))

    tempo_indication = Tempo(Duration(1, 4), 76)
    spacing_indication_2 = layouttools.SpacingIndication(
        tempo_indication, Duration(1, 68))

    assert spacing_indication_1 == spacing_indication_2


def test_layouttools_SpacingIndication___eq___02():
    r'''Spacing indications compare not equal when
    normalized spacing durations compare not equal.
    '''

    tempo_indication = Tempo(Duration(1, 8), 38)
    spacing_indication_1 = layouttools.SpacingIndication(
        tempo_indication, Duration(1, 68))

    tempo_indication = Tempo(Duration(1, 8), 38)
    spacing_indication_2 = layouttools.SpacingIndication(
        tempo_indication, Duration(1, 78))

    assert spacing_indication_1 != spacing_indication_2



# -*- encoding: utf-8 -*-
from abjad import *
import pytest


def test_scoretools_replace_contents_of_measures_in_expr_01():
    r'''Contents duration less than sum of duration of measures.
    Note spacer skip at end of second measure.
    '''

    staff = Staff(scoretools.make_spacer_skip_measures([(1, 8), (3, 16)]))

    r'''
    \new Staff {
        {
            \time 1/8
            s1 * 1/8
        }
        {
            \time 3/16
            s1 * 3/16
        }
    }
    '''

    notes = [Note("c'16"), Note("d'16"), Note("e'16"), Note("f'16")]
    scoretools.replace_contents_of_measures_in_expr(staff, notes)

    r'''
    \new Staff {
        {
            \time 1/8
            c'16
            d'16
        }
        {
            \time 3/16
            e'16
            f'16
            s1 * 1/16
        }
    }
    '''

    assert inspect(staff).is_well_formed()
    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 1/8
                c'16
                d'16
            }
            {
                \time 3/16
                e'16
                f'16
                s1 * 1/16
            }
        }
        '''
        )


def test_scoretools_replace_contents_of_measures_in_expr_02():
    r'''Some contents too big for some measures.
    Small measures skipped.
    '''

    staff = Staff(scoretools.make_spacer_skip_measures([(1, 16), (3, 16), (1, 16), (3, 16)]))

    r'''
    \new Staff {
        {
            \time 1/16
            s1 * 1/16
        }
        {
            \time 3/16
            s1 * 3/16
        }
        {
            \time 1/16
            s1 * 1/16
        }
        {
            \time 3/16
            s1 * 3/16
        }
    }
    '''

    notes = [Note("c'8"), Note("d'8")]
    scoretools.replace_contents_of_measures_in_expr(staff, notes)

    r'''
    \new Staff {
        {
            \time 1/16
            s1 * 1/16
        }
        {
            \time 3/16
            c'8
            s1 * 1/16
        }
        {
            \time 1/16
            s1 * 1/16
        }
        {
            \time 3/16
            d'8
            s1 * 1/16
        }
    }
    '''

    assert inspect(staff).is_well_formed()
    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 1/16
                s1 * 1/16
            }
            {
                \time 3/16
                c'8
                s1 * 1/16
            }
            {
                \time 1/16
                s1 * 1/16
            }
            {
                \time 3/16
                d'8
                s1 * 1/16
            }
        }
        '''
        )


def test_scoretools_replace_contents_of_measures_in_expr_03():
    r'''Raise MissingMeasureError when input expression
    contains no measures.
    '''

    note = Note("c'4")
    notes = [Note("c'8"), Note("d'8")]

    statement = 'scoretools.replace_contents_of_measures_in_expr(note, notes)'
    assert pytest.raises(MissingMeasureError, statement)


def test_scoretools_replace_contents_of_measures_in_expr_04():
    r'''Raise StopIteration when not enough measures.
    '''

    staff = Staff(scoretools.make_spacer_skip_measures([(1, 8), (1, 8)]))
    notes = [Note("c'16"), Note("d'16"), Note("e'16"), Note("f'16"), Note("g'16"), Note("a'16")]

    statement = 'scoretools.replace_contents_of_measures_in_expr(staff, notes)'
    assert pytest.raises(StopIteration, statement)


def test_scoretools_replace_contents_of_measures_in_expr_05():
    r'''Populate measures even when not enough total measures.
    '''

    staff = Staff(scoretools.make_spacer_skip_measures([(1, 8), (1, 8)]))
    scoretools.set_always_format_time_signature_of_measures_in_expr(staff)
    notes = [Note("c'16"), Note("d'16"), Note("e'16"), Note("f'16"), Note("g'16"), Note("a'16")]

    try:
        scoretools.replace_contents_of_measures_in_expr(staff, notes)
    except StopIteration:
        pass

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 1/8
                c'16
                d'16
            }
            {
                \time 1/8
                e'16
                f'16
            }
        }
        '''
        )

    assert inspect(staff).is_well_formed()


def test_scoretools_replace_contents_of_measures_in_expr_06():
    r'''Preserve ties.
    '''

    maker = rhythmmakertools.NoteRhythmMaker()
    durations = [(5, 16), (3, 16)]
    leaf_lists = maker(durations)
    leaves = sequencetools.flatten_sequence(leaf_lists)

    measures = scoretools.make_spacer_skip_measures(
        durations)
    staff = Staff(measures)
    measures = scoretools.replace_contents_of_measures_in_expr(staff, leaves)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 5/16
                c'4 ~
                c'16
            }
            {
                \time 3/16
                c'8.
            }
        }
        '''
        )

    assert inspect(staff).is_well_formed()



# -*- encoding: utf-8 -*-
# TODO: renamed to suggest_clef_for_named_pitches_in_expr
def suggest_clef_for_named_pitches(pitches):
    '''Suggest clef for named `pitches`:

    ::

        >>> staff = Staff(scoretools.make_notes(range(-12, -6), [(1, 4)]))
        >>> pitchtools.suggest_clef_for_named_pitches(staff)
        Clef('bass')

    Suggest clef based on minimal number of ledger lines.

    Returns clef.
    '''
    from abjad.tools import indicatortools
    from abjad.tools import pitchtools

    pitches = pitchtools.list_named_pitches_in_expr(pitches)

    diatonic_pitch_numbers = [pitch.diatonic_pitch_number for pitch in pitches]
    max_diatonic_pitch_number = max(diatonic_pitch_numbers)
    min_diatonic_pitch_number = min(diatonic_pitch_numbers)

    lowest_treble_line_pitch = pitchtools.NamedPitch('e', 4)
    lowest_treble_line_diatonic_pitch_number = \
        lowest_treble_line_pitch.diatonic_pitch_number
    candidate_steps_below_treble = \
        lowest_treble_line_diatonic_pitch_number - min_diatonic_pitch_number

    highest_bass_line_pitch = pitchtools.NamedPitch('a', 3)
    highest_bass_line_diatonic_pitch_number = \
        highest_bass_line_pitch.diatonic_pitch_number
    candidate_steps_above_bass = max_diatonic_pitch_number - highest_bass_line_diatonic_pitch_number

    if candidate_steps_above_bass < candidate_steps_below_treble:
        return indicatortools.Clef('bass')
    else:
        return indicatortools.Clef('treble')



# -*- encoding: utf-8 -*-


def label_leaves_in_expr_with_written_leaf_duration(
    expr, markup_direction=Down):
    r'''Label leaves in `expr` with writen leaf duration:

    ::

        >>> tuplet = Tuplet((2, 3), "c'8 d'8 e'8")
        >>> labeltools.label_leaves_in_expr_with_leaf_durations(tuplet)
        >>> print format(tuplet)
        \times 2/3 {
            c'8 _ \markup { \column { \small 1/8 \small 1/12 } }
            d'8 _ \markup { \column { \small 1/8 \small 1/12 } }
            e'8 _ \markup { \column { \small 1/8 \small 1/12 } }
        }

    ::

        >>> show(tuplet) # doctest: +SKIP

    Returns none.
    '''
    from abjad.tools import labeltools

    return labeltools.label_leaves_in_expr_with_leaf_durations(
        expr,
        label_durations=False,
        label_written_durations=True,
        markup_direction=markup_direction,
        )



# -*- encoding: utf-8 -*-
import sys
from abjad.tools.abctools import ContextManager


class RedirectedStreams(ContextManager):
    r'''A context manager for capturing stdout and stderr output.

    ::

        >>> import StringIO
        >>> string_io = StringIO.StringIO()
        >>> with systemtools.RedirectedStreams(stdout=string_io):
        ...     print "hello, world!"
        ...
        >>> result = string_io.getvalue()
        >>> string_io.close()
        >>> print result
        hello, world!

    Returns context manager.
    '''

    ### INITIALIZER ###

    def __init__(self, stdout=None, stderr=None):
        self._stdout = stdout or sys.stdout
        self._stderr = stderr or sys.stderr

    ### SPECIAL METHODS ###

    def __enter__(self):
        r'''Enters redirected streams context manager.

        Returns none.
        '''
        self.old_stdout, self.old_stderr = sys.stdout, sys.stderr
        self.old_stdout.flush(); self.old_stderr.flush()
        sys.stdout, sys.stderr = self._stdout, self._stderr

    def __exit__(self, exc_type, exc_value, traceback):
        r'''Exits redirected streams context manager.

        Returns none.
        '''
        self._stdout.flush(); self._stderr.flush()
        sys.stdout = self.old_stdout
        sys.stderr = self.old_stderr



# -*- encoding: utf-8 -*-
from abjad import *


def test_lilypondfiletools_HeaderBlock_01():

    header_block = lilypondfiletools.HeaderBlock()
    header_block.composer = markuptools.Markup('Josquin')
    header_block.title = markuptools.Markup('Missa sexti tonus')

    r'''
    \header {
        composer = \markup { Josquin }
        title = \markup { Missa sexti tonus }
    }
    '''

    assert systemtools.TestManager.compare(
        header_block,
        r'''
        \header {
            composer = \markup { Josquin }
            title = \markup { Missa sexti tonus }
        }
        '''
        )



# -*- encoding: utf-8 -*-
from abjad.tools import mathtools


def map_sequence_elements_to_numbered_sublists(sequence):
    '''Map `sequence` elements to numbered sublists:

    ::

        >>> sequencetools.map_sequence_elements_to_numbered_sublists([1, 2, -3, -4, 5])
        [[1], [2, 3], [-4, -5, -6], [-7, -8, -9, -10], [11, 12, 13, 14, 15]]

    ::

        >>> sequencetools.map_sequence_elements_to_numbered_sublists([1, 0, -3, -4, 5])
        [[1], [], [-2, -3, -4], [-5, -6, -7, -8], [9, 10, 11, 12, 13]]

    Note that numbering starts at ``1``.

    Returns newly constructed list of lists.
    '''

    if not isinstance(sequence, list):
        raise TypeError

    if not all(isinstance(x, (int, long)) for x in sequence):
        raise ValueError

    result = []
    current = 1

    for length in sequence:
        abs_length = abs(length)
        part = range(current, current + abs_length)
        part = [mathtools.sign(length) * x for x in part]
        result.append(part)
        current += abs_length

    return result



# -*- encoding: utf-8 -*-
from abjad.tools import mathtools


def yield_all_partitions_of_sequence(sequence):
    '''Yields all partitions of `sequence`.

    ::

        >>> sequence = [0, 1, 2, 3]
        >>> result = sequencetools.yield_all_partitions_of_sequence(sequence)
        >>> for partition in result:
        ...     partition
        ...
        [[0, 1, 2, 3]]
        [[0, 1, 2], [3]]
        [[0, 1], [2, 3]]
        [[0, 1], [2], [3]]
        [[0], [1, 2, 3]]
        [[0], [1, 2], [3]]
        [[0], [1], [2, 3]]
        [[0], [1], [2], [3]]

    Returns generator of newly created lists.
    '''

    # TODO: remove type restriction 
    if not isinstance(sequence, list):
        message = '{!r} must be list.'.format(sequence)
        raise TypeError(message)

    partitions = []

    len_l_minus_1 = len(sequence) - 1
    for i in range(2 ** len_l_minus_1):
        binary_string = mathtools.integer_to_binary_string(i)
        binary_string = binary_string.zfill(len_l_minus_1)
        part = sequence[0:1]
        partition = [part]
        for n, token in zip(sequence[1:], binary_string):
            if int(token) == 0:
                part.append(n)
            else:
                part = [n]
                partition.append(part)
        partitions.append(partition)

    return partitions



# -*- encoding: utf-8 -*-
import os
import types


class ImportManager(object):
    r'''Imports structured packages.
    '''

    ### PRIVATE METHODS ###

    @staticmethod
    def _get_public_function_names_in_module(module_file):
        r'''Collects and returns all public functions defined in
        module_file.
        '''
        result = []
        module_file = module_file.replace(os.sep, '.')
        mod = __import__(module_file, fromlist=['*'])
        for key, value in vars(mod).items():
            if not key.startswith('_'):
                # handle public function decorated with @require
                if getattr(value, 'func_closure', None):
                    module_name = getattr(value.func_closure[1].cell_contents,
                        '__module__', None)
                # handle plain old function
                else:
                    module_name = getattr(value, '__module__', None)
                if module_name == module_file:
                    result.append(value)
        return result

    @staticmethod
    def _import_contents_of_public_packages_in_path_into_namespace(
        path,
        namespace,
        ):
        r'''Inspects the top level of path.

        Finds public class packages and imports class package
        contents into namespace.

        Does not inspect lower levels of path.
        '''
        package_path = ImportManager._split_package_path(path)
        for name in os.listdir(path):
            fullname = os.path.join(path, name)
            if os.path.isdir(fullname):
                if name[0].isupper() and \
                    os.path.exists(os.path.join(fullname, '__init__.py')) and \
                    os.path.exists(os.path.join(fullname, '%s.py' % name)):
                    class_package = '.'.join([package_path, name])
                    class_module = '.'.join([class_package, name])
                    public_names = \
                        ImportManager._get_public_function_names_in_module(
                            class_module)
                    for public_name in public_names:
                        namespace[public_name.__name__] = public_name

    @staticmethod
    def _split_package_path(path):
        outer, inner = path, None
        while os.path.exists(os.path.join(outer, '__init__.py')):
            outer, inner = os.path.split(outer)
        package_path = os.path.relpath(path, outer)
        package_path = package_path.replace(os.sep, '.')
        return package_path

    ### PUBLIC METHODS ###

    @staticmethod
    def import_public_names_from_filesystem_path_into_namespace(
        path,
        namespace,
        delete_systemtools=True,
        **kwargs
        ):
        r'''Inspects the top level of `path`.

        Finds .py modules in path and imports public functions from
        .py modules into namespace.

        Finds packages in path and imports package names into namespace.

        Does not import package content into namespace.

        Does not inspect lower levels of path.
        '''
        package_path = ImportManager._split_package_path(path)
        for element in os.listdir(path):
            if os.path.isfile(os.path.join(path, element)):
                if not element.startswith('_') and element.endswith('.py'):
                    # import function inside module
                    name = os.path.splitext(element)[0]
                    submod = os.path.join(package_path, name)
                    functions = \
                        ImportManager._get_public_function_names_in_module(
                            submod)
                    for f in functions:
                        # handle public function decorated with @require
                        if f.__name__ == 'wrapper':
                            name = f.func_closure[1].cell_contents.__name__
                        else:
                            name = f.__name__
                        namespace[name] = f
            elif os.path.isdir(os.path.join(path, element)):
                if not element in ('.svn', '.git', 'test', '__pycache__'):
                    initializer_file_path = os.path.join(
                        path,
                        element,
                        '__init__.py',
                        )
                    if os.path.exists(initializer_file_path):
                        submod = '.'.join((package_path, element))
                        namespace[element] = __import__(submod, fromlist=['*'])
            else:
                message = 'neither a directory or file: {!r}'.format(element)
                raise ImportError(message)
        ImportManager._import_contents_of_public_packages_in_path_into_namespace(
            path, namespace)
        if delete_systemtools:
            if 'systemtools' in namespace:
                del(namespace['systemtools'])
        if ImportManager.__name__ in namespace:
            del(namespace[ImportManager.__name__])

    @staticmethod
    def import_structured_package(
        path,
        namespace,
        delete_systemtools=True,
        **kwargs
        ):
        r'''Imports public names from `path` into `namespace`.

        This is the custom function that all Abjad packages use to import
        public classes and functions on startup.

        The function will work for any package laid out like Abjad packages.
        '''
        ImportManager.import_public_names_from_filesystem_path_into_namespace(
            path,
            namespace,
            delete_systemtools=delete_systemtools,
            )
        if delete_systemtools:
            if 'systemtools' in namespace:
                del(namespace['systemtools'])
        if ImportManager.__name__ in namespace:
            del(namespace[ImportManager.__name__])



# -*- encoding: utf-8 -*-
from abjad import *


def test_lilypondfiletools_PaperBlock_minimal_page_breaking_01():

    paper_block = lilypondfiletools.PaperBlock()
    paper_block.minimal_page_breaking = True

    assert systemtools.TestManager.compare(
        paper_block,
        r'''
        \paper {
            #(define page-breaking ly:minimal-breaking)
        }
        '''
        )



# -*- encoding: utf-8 -*-
from abjad import *
import copy


def test_markuptools_Markup___copy___01():
    r'''Copy keywords.
    '''

    markup_1 = markuptools.Markup('foo bar', direction=Up, markup_name='foo')
    markup_2 = copy.copy(markup_1)

    assert markup_1 == markup_2
    assert repr(markup_1) == repr(markup_2)
    assert format(markup_1) == format(markup_2)



# -*- encoding: utf-8 -*-
from abjad.tools.pitchtools.PitchClassSegment import PitchClassSegment


class TwelveToneRow(PitchClassSegment):
    '''A twelve-tone row.

    ::

        >>> pitchtools.TwelveToneRow([0, 1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8])
        TwelveToneRow([0, 1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8])

    Twelve-tone rows validate pitch-classes at initialization.

    Twelve-tone rows inherit canonical operators from numbered
    pitch-class segment.

    Twelve-tone rows return numbered pitch-class segments
    on calls to getslice.

    Twelve-tone rows are immutable.
    '''

    ### CLASS VARIABLES ###

    _default_positional_input_arguments = (
        [0, 1, 11, 9, 3, 6, 7, 5, 4, 10, 2, 8],
        )

    ### INITIALIZER ###

    def __init__(self, tokens=None, custom_identifier=None):
        from abjad.tools import pitchtools
        assert tokens is not None
        PitchClassSegment.__init__(
            self,
            tokens=tokens,
            item_class=pitchtools.NumberedPitchClass,
            custom_identifier=custom_identifier,
            )
        self._validate_pitch_classes(self)

    ### SPECIAL METHODS ###

    def __getslice__(self, start, stop):
        r'''Gets items from `start` to `stop` in twelve-tone row.

        Returns pitch-class segment.
        '''
        from abjad.tools import pitchtools
        tokens = self._collection[start:stop]
        return PitchClassSegment(
            tokens=tokens,
            item_class=pitchtools.NumberedPitchClass,
            custom_identifier=self.custom_identifier,
            )

    def __makenew__(self, tokens=None, custom_identifier=None):
        r'''Makes new twelve-tone row with optional `tokens` and
        `custom_identifier`.

        Returns new twelve-tone row.
        '''
        from abjad.tools import pitchtools
        # allow for empty iterables:
        if tokens is None:
            tokens = self._collection
        custom_identifier = custom_identifier or self.custom_identifier
        return type(self)(
            tokens=tokens,
            custom_identifier=custom_identifier,
            )

    def __mul__(self, expr):
        r'''Multiplies twelve-tone row by `expr`.

        Returns pitch-class segment.
        '''
        return PitchClassSegment(self) * expr

    def __rmul__(self, expr):
        r'''Multiplies `expr` by twelve-tone row.

        Returns pitch-class segment.
        '''
        return PitchClassSegment(self) * expr

    ### PRIVATE PROPERTIES ###

    @property
    def _contents_string(self):
        return ', '.join([str(abs(pc)) for pc in self])

    ### PRIVATE METHODS ###

    @staticmethod
    def _validate_pitch_classes(pitch_classes):
        numbers = [abs(pc) for pc in pitch_classes]
        numbers.sort()
        if not numbers == range(12):
            message = 'must contain all twelve pitch-classes: {!r}.'
            message = message.format(pitch_classes)
            raise ValueError(message)

    ### PUBLIC METHODS ###d

    @classmethod
    def from_selection(
        cls, 
        selection, 
        item_class=None, 
        custom_identifier=None,
        ):
        r'''Makes twelve-tone row from `selection`.

        Returns twelve-tone row.
        '''
        raise NotImplementedError



# -*- encoding: utf-8 -*-
from abjad import *


def test_sequencetools_overwrite_sequence_elements_at_indices_01():

    sequence_1 = range(10)
    sequence_2 = sequencetools.overwrite_sequence_elements_at_indices(sequence_1, [(0, 3), (5, 3)])

    assert sequence_2 == [0, 0, 0, 3, 4, 5, 5, 5, 8, 9]


def test_sequencetools_overwrite_sequence_elements_at_indices_02():

    sequence_1 = range(10)
    sequence_2 = sequencetools.overwrite_sequence_elements_at_indices(sequence_1, [(0, 99)])

    assert sequence_2 == [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]



# -*- encoding: utf-8 -*-
from abjad import *


def test_labeltools_label_leaves_in_expr_with_numbered_intervals_01():

    staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    labeltools.label_leaves_in_expr_with_numbered_intervals(staff)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8 ^ \markup { +2 }
            d'8 ^ \markup { +2 }
            e'8 ^ \markup { +1 }
            f'8 ^ \markup { +2 }
            g'8 ^ \markup { +2 }
            a'8 ^ \markup { +2 }
            b'8 ^ \markup { +1 }
            c''8
        }
        '''
        )

    assert inspect(staff).is_well_formed()


def test_labeltools_label_leaves_in_expr_with_numbered_intervals_02():

    pitch_numbers = [0, 25, 11, -4, -14, -13, 9, 10, 6, 5]
    staff = Staff(scoretools.make_notes(pitch_numbers, [Duration(1, 8)]))
    labeltools.label_leaves_in_expr_with_numbered_intervals(staff)

    assert systemtools.TestManager.compare(
        staff,
        r"""
        \new Staff {
            c'8 ^ \markup { +25 }
            cs'''8 ^ \markup { -14 }
            b'8 ^ \markup { -15 }
            af8 ^ \markup { -10 }
            bf,8 ^ \markup { +1 }
            b,8 ^ \markup { +22 }
            a'8 ^ \markup { +1 }
            bf'8 ^ \markup { -4 }
            fs'8 ^ \markup { -1 }
            f'8
        }
        """
        )

    assert inspect(staff).is_well_formed()


def test_labeltools_label_leaves_in_expr_with_numbered_intervals_03():
    r'''Works with quartertones.
    '''

    pitch_numbers = [0, 25.5, 11.5, -4, -14, -13, 9, 10, 6.5, 5.5]
    staff = Staff(scoretools.make_notes(pitch_numbers, [Duration(1, 8)]))
    labeltools.label_leaves_in_expr_with_numbered_intervals(staff)

    assert systemtools.TestManager.compare(
        staff,
        r"""
        \new Staff {
            c'8 ^ \markup { +25.5 }
            dqf'''8 ^ \markup { -14 }
            bqs'8 ^ \markup { -15.5 }
            af8 ^ \markup { -10 }
            bf,8 ^ \markup { +1 }
            b,8 ^ \markup { +22 }
            a'8 ^ \markup { +1 }
            bf'8 ^ \markup { -3.5 }
            gqf'8 ^ \markup { -1 }
            fqs'8
        }
        """
        )

    assert inspect(staff).is_well_formed()



# -*- encoding: utf-8 -*-


def get_indices_of_sequence_elements_equal_to_true(sequence):
    '''Get indices of `sequence` elements equal to true:

    ::

        >>> sequence = [0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1]

    ::

        >>> sequencetools.get_indices_of_sequence_elements_equal_to_true(sequence)
        (3, 4, 5, 9, 10, 11, 12)

    Returns newly constructed tuple of zero or more nonnegative integers.
    '''

    result = []
    for i, x in enumerate(sequence):
        if bool(x):
            result.append(i)
    return tuple(result)



# -*- encoding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import selectiontools
from abjad.tools import sequencetools
from abjad.tools.topleveltools import mutate
Selection = selectiontools.Selection


def _rewrite_meter(
    components,
    meter,
    boundary_depth=None,
    initial_offset=None,
    maximum_dot_count=None,
    ):
    from abjad.tools import metertools

    assert isinstance(components, selectiontools.Selection), \
        repr(components)

    def get_offsets_at_depth(depth):
        if depth < len(offset_inventory):
            return offset_inventory[depth]
        while len(offset_inventory) <= depth:
            new_offsets = []
            old_offsets = offset_inventory[-1]
            for first, second in \
                sequencetools.iterate_sequence_pairwise_strict(old_offsets):
                new_offsets.append(first)
                new_offsets.append((first + second) / 2)
            new_offsets.append(old_offsets[-1])
            offset_inventory.append(tuple(new_offsets))
        return offset_inventory[depth]

    def is_acceptable_logical_tie(logical_tie_duration,
        logical_tie_starts_in_offsets,
        logical_tie_stops_in_offsets):
        #print '\tTESTING ACCEPTABILITY'
        if not logical_tie_duration.is_assignable:
            return False
        if maximum_dot_count is not None and \
            maximum_dot_count < logical_tie_duration.dot_count:
            return False
        if not logical_tie_starts_in_offsets and not logical_tie_stops_in_offsets:
            return False
        return True

    def is_boundary_crossing_logical_tie(
        logical_tie_start_offset, logical_tie_stop_offset):
        #print '\tTESTING BOUNDARY CROSSINGS'
        if boundary_depth is None:
            return False
        if not any(logical_tie_start_offset < x < logical_tie_stop_offset
            for x in boundary_offsets):
            return False
        if logical_tie_start_offset in boundary_offsets and \
            logical_tie_stop_offset in boundary_offsets:
            return False
        return True

    def recurse(logical_tie, depth=0):
        offsets = get_offsets_at_depth(depth)
        #print 'DEPTH:', depth

        logical_tie_duration = logical_tie._preprolated_duration
        logical_tie_start_offset = logical_tie.get_timespan().start_offset
        logical_tie_stop_offset = logical_tie.get_timespan().stop_offset
        logical_tie_starts_in_offsets = logical_tie_start_offset in offsets
        logical_tie_stops_in_offsets = logical_tie_stop_offset in offsets

        if not is_acceptable_logical_tie(
            logical_tie_duration,
            logical_tie_starts_in_offsets,
            logical_tie_stops_in_offsets):

            #print 'UNACCEPTABLE:', logical_tie, logical_tie_start_offset, logical_tie_stop_offset
            #print '\t', ' '.join([str(x) for x in offsets])
            split_offset = None
            offsets = get_offsets_at_depth(depth)

            # If the logical tie's start aligns, take the latest possible offset.
            if logical_tie_starts_in_offsets:
                offsets = reversed(offsets)

            for offset in offsets:
                if logical_tie_start_offset < offset < logical_tie_stop_offset:
                    split_offset = offset
                    break

            #print '\tABS:', split_offset
            if split_offset is not None:
                split_offset -= logical_tie_start_offset
                #print '\tREL:', split_offset
                #print ''
                shards = mutate(logical_tie[:]).split([split_offset])
                logical_ties = \
                    [selectiontools.LogicalTie(shard) for shard in shards]
                for logical_tie in logical_ties:
                    recurse(logical_tie, depth=depth)
            else:
                #print ''
                recurse(logical_tie, depth=depth + 1)

        elif is_boundary_crossing_logical_tie(
            logical_tie_start_offset,
            logical_tie_stop_offset):

            #print 'BOUNDARY CROSSING', logical_tie, logical_tie_start_offset, logical_tie_stop_offset
            offsets = boundary_offsets
            if logical_tie_start_offset in boundary_offsets:
                offsets = reversed(boundary_offsets)
            split_offset = None
            for offset in offsets:
                if logical_tie_start_offset < offset < logical_tie_stop_offset:
                    split_offset = offset
                    break
            assert split_offset is not None
            #print '\tABS:', split_offset
            split_offset -= logical_tie_start_offset
            #print '\tREL:', split_offset
            #print ''
            shards = mutate(logical_tie[:]).split([split_offset])
            logical_ties = \
                [selectiontools.LogicalTie(shard) for shard in shards]
            for logical_tie in logical_ties:
                recurse(logical_tie, depth=depth)

        else:
            #print 'ACCEPTABLE:', logical_tie, logical_tie_start_offset, logical_tie_stop_offset
            #print '\t', ' '.join([str(x) for x in offsets])
            #print ''
            logical_tie[:]._fuse()

    # Validate arguments.
    assert Selection._all_are_contiguous_components_in_same_logical_voice(
        components)
    if not isinstance(meter,
        metertools.Meter):
        meter = \
            metertools.Meter(meter)
    #assert sum([x._preprolated_duration for x in components]) == \
    #    meter.preprolated_duration
    if boundary_depth is not None:
        boundary_depth = int(boundary_depth)
    if maximum_dot_count is not None:
        maximum_dot_count = int(maximum_dot_count)
        assert 0 <= maximum_dot_count

    if initial_offset is None:
        initial_offset = durationtools.Offset(0)
    initial_offset = durationtools.Offset(initial_offset)

    # Build offset inventory, adjusted for initial offset and prolation.
    first_offset = components[0]._get_timespan().start_offset
    first_offset -= initial_offset
    prolation = components[0]._get_parentage(include_self=False).prolation
    offset_inventory = []
    for offsets in meter.depthwise_offset_inventory:
        offsets = [(x * prolation) + first_offset for x in offsets]
        offset_inventory.append(tuple(offsets))

    # Build boundary offset inventory, if applicable.
    if boundary_depth is not None:
        boundary_offsets = offset_inventory[boundary_depth]

    # Cache results of iterator, as we'll be mutating the underlying collection.
    items = tuple(_iterate_topmost_masked_logical_ties_rest_groups_and_containers_in_expr(components))
    for item in items:
        if isinstance(item, selectiontools.LogicalTie):
            #print 'RECURSING:', item
            recurse(item, depth=0)
        else:
            #print 'DESCENDING:', item
            preprolated_duration = sum([x._preprolated_duration for x in item])
            if preprolated_duration.numerator == 1:
                preprolated_duration = mathtools.NonreducedFraction(
                    preprolated_duration)
                preprolated_duration = preprolated_duration.with_denominator(
                    preprolated_duration.denominator * 4)
            sub_metrical_hierarchy = metertools.Meter(preprolated_duration)
            sub_boundary_depth = 1
            if boundary_depth is None:
                sub_boundary_depth = None
            _rewrite_meter(
                item[:],
                sub_metrical_hierarchy,
                boundary_depth=sub_boundary_depth,
                maximum_dot_count=maximum_dot_count,
                )


def _iterate_topmost_masked_logical_ties_rest_groups_and_containers_in_expr(
    expr):
    r'''Iterate topmost masked logical ties, rest groups and containers in
    `expr`, masked by `expr`:

    ::

        >>> input = "abj: | 2/4 c'4 d'4 ~ |"
        >>> input += "| 4/4 d'8. r16 r8. e'16 ~ 2/3 { e'8 ~ e'8 f'8 ~ } f'4 ~ |"
        >>> input += "| 4/4 f'8 g'8 ~ g'4 a'4 ~ a'8 b'8 ~ |"
        >>> input += "| 2/4 b'4 c''4 |"
        >>> staff = Staff(input)

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            {
                \time 2/4
                c'4
                d'4 ~
            }
            {
                \time 4/4
                d'8.
                r16
                r8.
                e'16 ~
                \times 2/3 {
                    e'8 ~
                    e'8
                    f'8 ~
                }
                f'4 ~
            }
            {
                f'8
                g'8 ~
                g'4
                a'4 ~
                a'8
                b'8 ~
            }
            {
                \time 2/4
                b'4
                c''4
            }
        }

    ::

        >>> from abjad.tools.metertools._rewrite_meter \
        ...     import _iterate_topmost_masked_logical_ties_rest_groups_and_containers_in_expr

    ::

        >>> for x in _iterate_topmost_masked_logical_ties_rest_groups_and_containers_in_expr(
        ...     staff[0]): x
        ...
        LogicalTie(Note("c'4"),)
        LogicalTie(Note("d'4"),)

    ::

        >>> for x in _iterate_topmost_masked_logical_ties_rest_groups_and_containers_in_expr(
        ...     staff[1]): x
        ...
        LogicalTie(Note("d'8."),)
        LogicalTie(Rest('r16'), Rest('r8.'))
        LogicalTie(Note("e'16"),)
        Tuplet(Multiplier(2, 3), "e'8 e'8 f'8")
        LogicalTie(Note("f'4"),)

    ::

        >>> for x in _iterate_topmost_masked_logical_ties_rest_groups_and_containers_in_expr(
        ...     staff[2]): x
        ...
        LogicalTie(Note("f'8"),)
        LogicalTie(Note("g'8"), Note("g'4"))
        LogicalTie(Note("a'4"), Note("a'8"))
        LogicalTie(Note("b'8"),)

    ::

        >>> for x in _iterate_topmost_masked_logical_ties_rest_groups_and_containers_in_expr(
        ...     staff[3]): x
        ...
        LogicalTie(Note("b'4"),)
        LogicalTie(Note("c''4"),)

    Returns generator.
    '''
    from abjad.tools import scoretools
    from abjad.tools import selectiontools
    from abjad.tools import spannertools

    last_tie_spanner = None
    current_leaf_group = None
    current_leaf_group_is_silent = False

    for x in expr:
        if isinstance(x, (scoretools.Note, scoretools.Chord)):
            this_tie_spanner = x._get_spanners(spannertools.Tie) or None
            if current_leaf_group is None:
                current_leaf_group = []
            elif current_leaf_group_is_silent or \
                this_tie_spanner is None or \
                last_tie_spanner != this_tie_spanner:
                yield selectiontools.LogicalTie(current_leaf_group)
                current_leaf_group = []
            current_leaf_group_is_silent = False
            current_leaf_group.append(x)
            last_tie_spanner = this_tie_spanner
        elif isinstance(x, (scoretools.Rest, scoretools.Skip)):
            if current_leaf_group is None:
                current_leaf_group = []
            elif not current_leaf_group_is_silent:
                yield selectiontools.LogicalTie(current_leaf_group)
                current_leaf_group = []
            current_leaf_group_is_silent = True
            current_leaf_group.append(x)
            last_tie_spanner = None
        elif isinstance(x, scoretools.Container):
            if current_leaf_group is not None:
                yield selectiontools.LogicalTie(current_leaf_group)
                current_leaf_group = None
                last_tie_spanner = None
            yield x

        else:
            message = 'unhandled component: {!r}.'.format(x)
            raise Exception(message)
    if current_leaf_group is not None:
        yield selectiontools.LogicalTie(current_leaf_group)



# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_sequencetools_map_sequence_elements_to_canonic_tuples_01():

    sequence_1 = range(10)
    sequence_2 = sequencetools.map_sequence_elements_to_canonic_tuples(sequence_1)

    assert sequence_2 == [(0,), (1,), (2,), (3,), (4,), (4, 1), (6,), (7,), (8,), (8, 1)]


def test_sequencetools_map_sequence_elements_to_canonic_tuples_02():

    sequence_1 = range(10)
    false = sequencetools.map_sequence_elements_to_canonic_tuples(
        sequence_1, decrease_parts_monotonically=False)

    assert false == [(0,), (1,), (2,), (3,), (4,), (1, 4), (6,), (7,), (8,), (1, 8)]


def test_sequencetools_map_sequence_elements_to_canonic_tuples_03():
    r'''Raise TypeError when sequence_1 is not a list.
    Raise ValueError on noninteger elements in sequence_1.
    '''

    statement = "sequencetools.map_sequence_elements_to_canonic_tuples('foo')"
    assert pytest.raises(TypeError, statement)
    statement = 'sequencetools.map_sequence_elements_to_canonic_tuples(' + \
        '[Fraction(1, 2), Fraction(1, 2)])'
    assert pytest.raises(ValueError, statement)



# -*- encoding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import detach
from abjad.tools.topleveltools import iterate


def move_full_measure_tuplet_prolation_to_measure_time_signature(expr):
    r'''Move prolation of full-measure tuplet to time signature of measure.

    Measures usually become non-power-of-two as as result:

    ::

        >>> t = Measure((2, 8), [scoretools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")])
        >>> scoretools.move_full_measure_tuplet_prolation_to_measure_time_signature(t)

    ..  doctest::

        >>> print format(t)
        {
            \time 3/12
            \scaleDurations #'(2 . 3) {
                c'8
                d'8
                e'8
            }
        }

    Returns none.
    '''
    from abjad.tools import scoretools
    from abjad.tools import indicatortools

    for measure in iterate(expr).by_class(scoretools.Measure):
        if len(measure) == 1:
            if isinstance(measure[0], scoretools.Tuplet):
                tuplet = measure[0]
                tuplet_multiplier = tuplet.multiplier
                tuplet_denominator = tuplet_multiplier.denominator
                reduced_denominator = mathtools.remove_powers_of_two(tuplet_denominator)
                time_signature = measure.time_signature
                time_signature_rational = durationtools.Duration(
                    time_signature.numerator, time_signature.denominator)
                numerator = time_signature_rational.numerator * reduced_denominator
                denominator = time_signature_rational.denominator * reduced_denominator
                time_signature = indicatortools.TimeSignature((numerator, denominator))
                detach(indicatortools.TimeSignature, measure)
                attach(time_signature, measure)
                time_signature_multiplier = \
                    measure.time_signature.implied_prolation
                written_adjustment = tuplet_multiplier / time_signature_multiplier
                tuplet._extract()
                measure._scale_contents(written_adjustment)



# -*- encoding: utf-8 -*-
import collections
from abjad.tools import durationtools
from abjad.tools.abctools import AbjadObject


class MetricAccentKernel(AbjadObject):
    r'''A metrical kernel, or offset-impulse-response-filter.

    ::

        >>> hierarchy = metertools.Meter((5, 8))
        >>> kernel = hierarchy.generate_offset_kernel_to_denominator(8)
        >>> kernel
        MetricAccentKernel(
            {
                Offset(0, 1): Multiplier(3, 11),
                Offset(1, 8): Multiplier(1, 11),
                Offset(1, 4): Multiplier(1, 11),
                Offset(3, 8): Multiplier(2, 11),
                Offset(1, 2): Multiplier(1, 11),
                Offset(5, 8): Multiplier(3, 11),
                }
            )

    Call the kernel against an expression from which offsets can be counted
    to receive an impulse-response:

    ::

        >>> offsets = [(0, 8), (1, 8), (1, 8), (3, 8)]
        >>> kernel(offsets)
        0.6363636363636364

    Return `MetricAccentKernel` instance.
    '''
    ### CLASS VARIABLES ###

    __slots__ = (
        '_kernel',
        '_offsets',
        )

    ### INITIALIZER ###

    def __init__(self, kernel=None):
        kernel = kernel or {}
        assert isinstance(kernel, dict)
        #assert 1 < len(kernel)
        for key, value in kernel.iteritems():
            assert isinstance(key, durationtools.Offset)
            assert isinstance(value, durationtools.Multiplier)
        self._kernel = kernel.copy()
        self._offsets = tuple(sorted(self._kernel))

    ### SPECIAL METHODS ###

    def __call__(self, expr):
        r'''Calls metrical accent kernal on `expr`.

        Returns float.
        '''
        offset_count = self.count_offsets_in_expr(expr)
        response = 0.
        for offset, count in offset_count.iteritems():
            if offset in self._kernel:
                response += (self._kernel[offset] * count)
        return float(response)

    def __eq__(self, expr):
        r'''True when `expr` is a metrical accent kernal with a kernal equal to
        that of this metrical accent kernel. Otherwise false.

        Returns boolean.
        '''
        if type(self) == type(expr):
            if self._kernel == expr._kernel:
                return True
        return False

    ### PRIVATE PROPERTIES ###

    @property
    def _repr_specification(self):
        return self._storage_format_specification

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        positional_argument_values = (
            self.kernel,
            )
        return systemtools.StorageFormatSpecification(
            self,
            keyword_argument_names=(),
            positional_argument_values=positional_argument_values,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def kernel(self):
        r'''The kernel datastructure.

        Returns dict.
        '''
        return self._kernel.copy()

    ### PUBLIC METHODS ###

    @staticmethod
    def count_offsets_in_expr(expr):
        r'''Count offsets in `expr`.

        ..  container:: example

            **Example 1:**

            ::

                >>> score = Score()
                >>> score.append(Staff("c'4. d'8 e'2"))
                >>> score.append(Staff(r'\clef bass c4 b,4 a,2'))

            ..  doctest::

                >>> print format(score)
                \new Score <<
                    \new Staff {
                        c'4.
                        d'8
                        e'2
                    }
                    \new Staff {
                        \clef "bass"
                        c4
                        b,4
                        a,2
                    }
                >>

            ::

                >>> show(score) # doctest: +SKIP

            ::

                >>> MetricAccentKernel = metertools.MetricAccentKernel
                >>> leaves = score.select_leaves(
                ...     allow_discontiguous_leaves=True)
                >>> counter = MetricAccentKernel.count_offsets_in_expr(leaves)
                >>> for offset, count in sorted(counter.items()):
                ...     offset, count
                ...
                (Offset(0, 1), 2)
                (Offset(1, 4), 2)
                (Offset(3, 8), 2)
                (Offset(1, 2), 4)
                (Offset(1, 1), 2)

        ..  container:: example

            **Example 2:**

            ::

                >>> a = timespantools.Timespan(0, 10)
                >>> b = timespantools.Timespan(5, 15)
                >>> c = timespantools.Timespan(15, 20)

            ::

                >>> counter = MetricAccentKernel.count_offsets_in_expr((a, b, c))
                >>> for offset, count in sorted(counter.items()):
                ...     offset, count
                ...
                (Offset(0, 1), 1)
                (Offset(5, 1), 1)
                (Offset(10, 1), 1)
                (Offset(15, 1), 2)
                (Offset(20, 1), 1)

        Returns counter.
        '''
        counter = collections.Counter()
        for x in expr:
            if hasattr(x, 'start_offset') and hasattr(x, 'stop_offset'):
                counter[x.start_offset] += 1
                counter[x.stop_offset] += 1
            elif hasattr(x, '_get_timespan'):
                counter[x._get_timespan().start_offset] += 1
                counter[x._get_timespan().stop_offset] += 1
            # TODO: remove this branch in favor of the _get_timespan above
            elif hasattr(x, 'get_timespan'):
                counter[x.get_timespan().start_offset] += 1
                counter[x.get_timespan().stop_offset] += 1
            else:
                offset = durationtools.Offset(x)
                counter[offset] += 1
        return counter



# -*- encoding: utf-8 -*-
from abjad.tools import mathtools


def merge_duration_sequences(*sequences):
    r'''Merge duration `sequences`:

    ::

        >>> sequencetools.merge_duration_sequences([10, 10, 10], [7])
        [7, 3, 10, 10]

    Merge more duration sequences:

    ::

        >>> sequencetools.merge_duration_sequences([10, 10, 10], [10, 10])
        [10, 10, 10]

    The idea is that each sequence element represents a duration.

    Returns list.
    '''
    from abjad.tools import sequencetools

    offset_lists = []
    for sequence in sequences:
        offset_list = mathtools.cumulative_sums(sequence, start=None)
        offset_lists.append(offset_list)

    all_offsets = sequencetools.join_subsequences(offset_lists)
    all_offsets = list(sorted(set(all_offsets)))
    all_offsets.insert(0, 0)
    all_durations = mathtools.difference_series(all_offsets)

    return all_durations



# -*- encoding: utf-8 -*-
from abjad.tools import abctools


class LilyPondDimension(abctools.AbjadObject):
    r'''A LilyPond page dimension.

    ::

        >>> dimension = lilypondfiletools.LilyPondDimension(2, 'in')

    ..  doctest::

        >>> print format(dimension)
        2.0\in

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_unit', 
        '_value',
        )

    ### INITIALIZER ###

    def __init__(self, value=0, unit='cm'):
        assert 0 <= value
        assert unit in ('cm', 'in', 'mm')
        self._value = float(value)
        self._unit = unit

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats LilyPond dimension.

        Returns string.
        '''
        if format_specification in ('', 'lilypond'):
            return self._lilypond_format
        return str(self)

    ### PRIVATE PROPERTIES ###

    @property
    def _format_pieces(self):
        return [r'{}\{}'.format(self.value, self.unit)]

    @property
    def _lilypond_format(self):
        return '\n'.join(self._format_pieces)

    ### PUBLIC PROPERTIES ###

    @property
    def unit(self):
        r'''Unit of LilyPond dimension.

        Returns string.
        '''
        return self._unit

    @property
    def value(self):
        r'''Value of LilyPond dimension.

        Returns float.
        '''
        return self._value



# -*- encoding: utf-8 -*-
import select
import subprocess
import time
from abjad.tools import abctools


class Pipe(abctools.AbjadObject, subprocess.Popen):
    r'''A two-way, non-blocking pipe for interprocess communication:

    ::

        >>> pipe = documentationtools.Pipe('python', ['-i'])
        >>> pipe.writeline('my_list = [1, 2, 3]') # doctest: +SKIP
        >>> pipe.writeline('print my_list')       # doctest: +SKIP

    Return `Pipe` instance.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_arguments', 
        '_executable', 
        '_timeout',
        )

    ### INITIALIZER ###

    def __init__(self, executable='python', arguments=('-i',), timeout=0):
        self._arguments = arguments
        self._executable = executable
        self._timeout = timeout
        argv = [executable]
        if arguments != None:
            argv = argv + list(arguments)
        subprocess.Popen.__init__(self, argv,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT)

    ### PUBLIC PROPERTIES ###

    @property
    def arguments(self):
        r'''Arguments of pipe.
        '''
        return self._arguments

    @property
    def executable(self):
        r'''Executable of pipe.
        '''
        return self._executable

    @property
    def timeout(self):
        r'''Timeout of pipe.
        '''
        return self._timeout

    ### PRIVATE METHODS ###

    def _readbyte(self, n=1):
        poll = select.poll()
        poll.register(self.stdout.fileno(), select.POLLIN or select.POLLPRI)
        fd = poll.poll(self.timeout)
        if len(fd):
            f = fd[0]
            if f[1] > 0:
                return self.stdout.read(n)

    ### PUBLIC METHODS ###

    def close(self):
        r'''Closes pipe.
        '''
        self.terminate()
        self.wait()

    def read(self):
        r'''Reads from pipe.
        '''
        c = self._readbyte()
        string = ""
        while c != None:
            string = string + str(c)
            c = self._readbyte()
        return string

    def read_wait(self, seconds=0.01):
        r'''Tries to read from pipe.  Wait `seconds` 
        if nothing comes out and the repeats.

        Should be used with caution, as this may loop forever.
        '''
        while True:
            read = self.read()
            if read:
                return read
            time.sleep(seconds)

    def write(self, data):
        r'''Writes `data` into pipe.
        '''
        poll = select.poll()
        poll.register(self.stdin.fileno(), select.POLLOUT)
        fd = poll.poll(self.timeout)
        if len(fd):
            f = fd[0]
            if f[1] > 0:
                self.stdin.write(data)

    def write_line(self, data):
        r'''Write `data` into pipe. Then writes newline to pipe.
        '''
        self.write('%s\n' % data)



# -*- encoding: utf-8 -*-
from abjad.tools import pitchtools
from abjad.tools.spannertools.Spanner import Spanner


class TrillSpanner(Spanner):
    r'''A trill spanner.

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> trill = spannertools.TrillSpanner()
        >>> attach(trill, staff[:])
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            c'8 \startTrillSpan
            d'8
            e'8
            f'8 \stopTrillSpan
        }

    '''

    ### INITIALIZER ###

    def __init__(
        self, 
        components=None,
        overrides=None,
        ):
        Spanner.__init__(
            self, 
            components,
            overrides=overrides,
            )
        self._pitch = None

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        new.written_pitch = self.written_pitch

    def _format_before_leaf(self, leaf):
        result = []
        if self.pitch is not None:
            if self._is_my_first_leaf(leaf):
                result.append(r'\pitchedTrill')
        return result

    def _format_right_of_leaf(self, leaf):
        result = []
        if self._is_my_first_leaf(leaf):
            result.append(r'\startTrillSpan')
            if self.pitch is not None:
                result.append(str(self.pitch))
        if self._is_my_last_leaf(leaf):
            result.append(r'\stopTrillSpan')
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def pitch(self):
        r'''Gets and sets optional pitch of trill spanner.

            ::

                >>> t = Staff("c'8 d'8 e'8 f'8")
                >>> trill = spannertools.TrillSpanner()
                >>> attach(trill, t[:2])
                >>> trill.pitch = NamedPitch('cs', 4)

            ..  doctest::

                >>> print format(t)
                \new Staff {
                    \pitchedTrill c'8 \startTrillSpan cs'
                    d'8 \stopTrillSpan
                    e'8
                    f'8
                }

        Returns pitch or none.
        '''
        return self._pitch

    @pitch.setter
    def pitch(self, expr):
        if expr is None:
            self._pitch = expr
        else:
            pitch = pitchtools.NamedPitch(expr)
            self._pitch = pitch

    @property
    def written_pitch(self):
        r'''Gets and sets written pitch of trill spanner.

        Returns pitch.
        '''
        return self.pitch

    @written_pitch.setter
    def written_pitch(self, arg):
        self.pitch = arg



# -*- encoding: utf-8 -*-
from abjad.tools import selectiontools


def make_spacer_skip_measures(time_signatures):
    r'''Make measures with full-measure spacer skips from `time_signatures`.

    ..  container:: example

        ::

            >>> measures = scoretools.make_spacer_skip_measures(
            ...     [(1, 8), (5, 16), (5, 16)])
            >>> staff = Staff(measures)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print format(staff)
            \new Staff {
                {
                    \time 1/8
                    s1 * 1/8
                }
                {
                    \time 5/16
                    s1 * 5/16
                }
                {
                    s1 * 5/16
                }
            }

    Returns selection of unincorporated measures.
    '''
    from abjad.tools import indicatortools
    from abjad.tools import scoretools

    # check input
    time_signatures = [
        indicatortools.TimeSignature(x) for x in time_signatures]

    # make measures
    measures = [scoretools.Measure(x, []) for x in time_signatures]
    scoretools.fill_measures_in_expr_with_full_measure_spacer_skips(measures)

    # return measures
    measures = selectiontools.Selection(measures)
    return measures



# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_mathtools_divide_number_by_ratio_01():
    r'''Divide int by ratio.
    '''

    duration = mathtools.divide_number_by_ratio(1, [1, 1, 2])

    assert len(duration) == 3
    assert duration[0] == Duration(1, 4)
    assert duration[1] == Duration(1, 4)
    assert duration[2] == Duration(1, 2)


def test_mathtools_divide_number_by_ratio_02():
    r'''Divide fraction by ratio.
    '''

    duration = mathtools.divide_number_by_ratio(Duration(1, 2), [1, 1, 2])

    assert len(duration) == 3
    assert duration[0] == Duration(1, 8)
    assert duration[1] == Duration(1, 8)
    assert duration[2] == Duration(1, 4)


def test_mathtools_divide_number_by_ratio_03():
    r'''Raise type error on nonnumber.
    '''

    statement = "mathtools.divide_number_by_ratio('foo', [1, 1, 3])"
    assert pytest.raises(Exception, statement)


def test_mathtools_divide_number_by_ratio_04():
    r'''Duration returns durations.
    '''

    result = mathtools.divide_number_by_ratio(Duration(1, 4), [1, 1, 1])
    assert result == [Duration(1, 12), Duration(1, 12), Duration(1, 12)]



# -*- encoding: utf-8 -*-
import pytest
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_lilypondparsertools_LilyPondParser__spanners__Text_01():
    r'''Successful text spanners, showing single leaf overlap.
    '''

    target = Container(scoretools.make_notes([0] * 4, [(1, 4)]))
    text_spanner = spannertools.TextSpanner()
    attach(text_spanner, target[2:])
    text_spanner = spannertools.TextSpanner()
    attach(text_spanner, target[:3])

    assert systemtools.TestManager.compare(
        target,
        r'''
        {
            c'4 \startTextSpan
            c'4
            c'4 \stopTextSpan \startTextSpan
            c'4 \stopTextSpan
        }
        '''
        )

    parser = LilyPondParser()
    result = parser(format(target))
    assert format(target) == format(result) and target is not result


def test_lilypondparsertools_LilyPondParser__spanners__Text_02():
    r'''Swapped start and stop.
    '''

    target = Container(scoretools.make_notes([0] * 4, [(1, 4)]))
    text_spanner = spannertools.TextSpanner()
    attach(text_spanner, target[2:])
    text_spanner = spannertools.TextSpanner()
    attach(text_spanner, target[:3])

    assert systemtools.TestManager.compare(
        target,
        r'''
        {
            c'4 \startTextSpan
            c'4
            c'4 \stopTextSpan \startTextSpan
            c'4 \stopTextSpan
        }
        '''
        )

    string = r"\relative c' { c \startTextSpan c c \startTextSpan \stopTextSpan c \stopTextSpan }"

    parser = LilyPondParser()
    result = parser(string)
    assert format(target) == format(result) and target is not result


def test_lilypondparsertools_LilyPondParser__spanners__Text_03():
    r'''Single leaf.
    '''

    string = r'{ c \startTextSpan \stopTextSpan c c c }'
    assert pytest.raises(Exception, 'LilyPondParser()(string)')


def test_lilypondparsertools_LilyPondParser__spanners__Text_04():
    r'''Unterminated.
    '''

    string = r'{ c \startTextSpan c c c }'
    assert pytest.raises(Exception, 'LilyPondParser()(string)')


def test_lilypondparsertools_LilyPondParser__spanners__Text_05():
    r'''Unstarted.
    '''

    string = r'{ c c c c \stopTextSpan }'
    assert pytest.raises(Exception, 'LilyPondParser()(string)')


def test_lilypondparsertools_LilyPondParser__spanners__Text_06():
    r'''Nested.
    '''

    string = r'{ c \startTextSpan c \startTextSpan c \stopTextSpan c \stopTextSpan }'
    assert pytest.raises(Exception, 'LilyPondParser()(string)')



# -*- encoding: utf-8 -*-
from abjad import *


def test_pitchtools_contains_subsegment_01():
    r'''True when all pcs appear in octave order in pitches.
    '''

    pcs = [2, 7, 10]
    pitches = [6, 9, 12, 13, 14, 19, 22, 27, 28, 29, 32, 35]
    assert pitchtools.contains_subsegment(pcs, pitches)


def test_pitchtools_contains_subsegment_02():
    r'''True when all pcs appear in octave order in pitches.
    '''

    pcs = [2, 3, 4]
    pitches = [6, 9, 12, 13, 14, 19, 22, 27, 28, 29, 32, 35]
    assert not pitchtools.contains_subsegment(pcs, pitches)



# -*- encoding: utf-8 -*-
from abjad.tools.datastructuretools.PayloadTree import PayloadTree


class CyclicPayloadTree(PayloadTree):
    r'''A cyclic payload tree.

    Abjad data structure to work with a sequence whose elements have been
    grouped into arbitrarily many levels of cyclic containment.

    Exactly like the ``PayloadTree`` class but with the additional affordance
    that all integer indices of any size work at every level of structure.

    Cyclic pyaload trees raise no index errors.

    Here is a cyclic payload tree:

    ::

        >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
        >>> tree = datastructuretools.CyclicPayloadTree(sequence)

    ::

        >>> tree
        CyclicPayloadTree([[0, 1], [2, 3], [4, 5], [6, 7]])

    Here's an internal node:

    ::

        >>> tree[2]
        CyclicPayloadTree([4, 5])

    Here's the same node indexed with a different way:

    ::

        >>> tree[2]
        CyclicPayloadTree([4, 5])

    With a negative index:

    ::

        >>> tree[-2]
        CyclicPayloadTree([4, 5])

    And another negative index:

    ::

        >>> tree[-6]
        CyclicPayloadTree([4, 5])

    Here's a leaf node:

    ::

        >>> tree[2][0]
        CyclicPayloadTree(4)

    And here's the same node indexed a different way:

    ::

        >>> tree[2][20]
        CyclicPayloadTree(4)

    All other interface attributes function as in ``PayloadTree``.
    '''

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats cyclic tree.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)

    def __iter__(self):
        r'''Iterates cyclic payload tree.

        ::

            >>> for x in tree:
            ...     x
            CyclicPayloadTree([0, 1])
            CyclicPayloadTree([2, 3])
            CyclicPayloadTree([4, 5])
            CyclicPayloadTree([6, 7])

        Yields cyclic payload tree nodes.
        '''
        return self._noncyclic_children.__iter__()

    ### PRVATE METHODS ###

    def _initialize_children_list(self):
        from abjad.tools import datastructuretools
        return datastructuretools.CyclicList([])

    ### PUBLIC METHODS ###

    def get_next_n_nodes_at_level(self, n, level):
        r'''Gets next `n` nodes of cyclic payload tree at `level`.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.CyclicPayloadTree(sequence)

        Gets next 4 nodes at level 2:

        ::

            >>> for x in tree[0][0].get_next_n_nodes_at_level(4, 2):
            ...     x
            CyclicPayloadTree(1)
            CyclicPayloadTree(2)
            CyclicPayloadTree(3)
            CyclicPayloadTree(4)

        Gets next 10 nodes at level 2:

        ::

            >>> for node in tree[0][0].get_next_n_nodes_at_level(10, 2):
            ...     node
            CyclicPayloadTree(1)
            CyclicPayloadTree(2)
            CyclicPayloadTree(3)
            CyclicPayloadTree(4)
            CyclicPayloadTree(5)
            CyclicPayloadTree(6)
            CyclicPayloadTree(7)
            CyclicPayloadTree(1)
            CyclicPayloadTree(2)
            CyclicPayloadTree(3)

        ### PREVIOUS MNODES ###

        Gets previous 4 nodes at level 2:

        ::

            >>> for x in tree[0][0].get_next_n_nodes_at_level(-4, 2):
            ...     x
            CyclicPayloadTree(7)
            CyclicPayloadTree(6)
            CyclicPayloadTree(5)
            CyclicPayloadTree(4)

        Gets previous 10 nodes at level 2:

        ::

            >>> for node in tree[0][0].get_next_n_nodes_at_level(-10, 2):
            ...     node
            CyclicPayloadTree(7)
            CyclicPayloadTree(6)
            CyclicPayloadTree(5)
            CyclicPayloadTree(4)
            CyclicPayloadTree(3)
            CyclicPayloadTree(2)
            CyclicPayloadTree(1)
            CyclicPayloadTree(7)
            CyclicPayloadTree(6)
            CyclicPayloadTree(5)

        Returns list of nodes.
        '''
        return PayloadTree.get_next_n_nodes_at_level(self, n, level)

    def get_node_at_position(self, position):
        r'''Gets cyclic payload tree node at `position`.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.CyclicPayloadTree(sequence)

        ::

            >>> tree.get_node_at_position((2, 1))
            CyclicPayloadTree(5)

        ::

            >>> tree.get_node_at_position((2, 99))
            CyclicPayloadTree(5)

        ::

            >>> tree.get_node_at_position((82, 1))
            CyclicPayloadTree(5)

        ::

            >>> tree.get_node_at_position((82, 99))
            CyclicPayloadTree(5)

        Returns node.
        '''
        return PayloadTree.get_node_at_position(self, position)

    def iterate_forever_depth_first(self, reverse=False):
        r'''Iterate cyclic payload tree tree depth first.

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.CyclicPayloadTree(sequence)

        ..  container:: example
        
            **Example 1.** Iterates from left to right:

            ::

                >>> generator = tree.iterate_forever_depth_first()
                >>> for i in range(20):
                ...     generator.next()
                CyclicPayloadTree([[0, 1], [2, 3], [4, 5], [6, 7]])
                CyclicPayloadTree([0, 1])
                CyclicPayloadTree(0)
                CyclicPayloadTree(1)
                CyclicPayloadTree([2, 3])
                CyclicPayloadTree(2)
                CyclicPayloadTree(3)
                CyclicPayloadTree([4, 5])
                CyclicPayloadTree(4)
                CyclicPayloadTree(5)
                CyclicPayloadTree([6, 7])
                CyclicPayloadTree(6)
                CyclicPayloadTree(7)
                CyclicPayloadTree([[0, 1], [2, 3], [4, 5], [6, 7]])
                CyclicPayloadTree([0, 1])
                CyclicPayloadTree(0)
                CyclicPayloadTree(1)
                CyclicPayloadTree([2, 3])
                CyclicPayloadTree(2)
                CyclicPayloadTree(3)

        ..  container:: example

            **Example 2.** Iterates from right to left:

            ::

                >>> generator = tree.iterate_forever_depth_first(
                ...     reverse=True)
                >>> for i in range(20):
                ...     generator.next()
                CyclicPayloadTree([[0, 1], [2, 3], [4, 5], [6, 7]])
                CyclicPayloadTree([6, 7])
                CyclicPayloadTree(7)
                CyclicPayloadTree(6)
                CyclicPayloadTree([4, 5])
                CyclicPayloadTree(5)
                CyclicPayloadTree(4)
                CyclicPayloadTree([2, 3])
                CyclicPayloadTree(3)
                CyclicPayloadTree(2)
                CyclicPayloadTree([0, 1])
                CyclicPayloadTree(1)
                CyclicPayloadTree(0)
                CyclicPayloadTree([[0, 1], [2, 3], [4, 5], [6, 7]])
                CyclicPayloadTree([6, 7])
                CyclicPayloadTree(7)
                CyclicPayloadTree(6)
                CyclicPayloadTree([4, 5])
                CyclicPayloadTree(5)
                CyclicPayloadTree(4)

        Yields cyclic payload tree nodes.
        '''
        while True:
            for node in PayloadTree.iterate_depth_first(self, reverse=reverse):
                yield node
